@@ -30,6 +30,7 @@
 #include <drogon/HttpRequest.h>
 #include <drogon/HttpResponse.h>
 #include <drogon/orm/DbClient.h>
+#include <drogon/orm/DbConfig.h>
 #include <drogon/nosql/RedisClient.h>
 #include <drogon/Cookie.h>
 #include <trantor/net/Resolver.h>
@@ -42,6 +43,16 @@
 #include <vector>
 #include <chrono>
 
+#if defined(__APPLE__) && defined(__MACH__) && \
+    (defined(__ENVIRONMENT_IPHONE_OS__) ||     \
+     defined(__IPHONE_OS_VERSION_MIN_REQUIRED))
+// iOS
+#define TARGET_OS_IOS 1
+#else
+// not iOS
+#define TARGET_OS_IOS 0
+#endif
+
 namespace drogon
 {
 // the drogon banner
@@ -66,6 +77,8 @@ using ExceptionHandler =
 using DefaultHandler =
     std::function<void(const HttpRequestPtr &,
                        std::function<void(const HttpResponsePtr &)> &&)>;
+using HttpHandlerInfo = std::tuple<std::string, HttpMethod, std::string>;
+
 #ifdef __cpp_impl_coroutine
 class HttpAppFramework;
 
@@ -144,7 +157,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      */
     virtual trantor::EventLoop *getLoop() const = 0;
 
-    /// Get an IO loop with id. E.g. 0 <= id < #Total thread-loops
+    /// Get an IO loop with id. E.g. 0 <= id < \#Total thread-loops
     /**
      * @note
      * The event loop is one of the network IO loops. Use the loop
@@ -171,7 +184,19 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * be sent to the client to provide a custom layout.
      */
     virtual HttpAppFramework &setCustomErrorHandler(
-        std::function<HttpResponsePtr(HttpStatusCode)> &&resp_generator) = 0;
+        std::function<HttpResponsePtr(HttpStatusCode,
+                                      const HttpRequestPtr &req)>
+            &&resp_generator) = 0;
+
+    HttpAppFramework &setCustomErrorHandler(
+        std::function<HttpResponsePtr(HttpStatusCode)> &&resp_generator)
+    {
+        return setCustomErrorHandler(
+            [cb = std::move(resp_generator)](HttpStatusCode code,
+                                             const HttpRequestPtr &) {
+                return cb(code);
+            });
+    }
 
     /// Get custom error handler
     /**
@@ -179,8 +204,9 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * setCustomErrorHandler. If none was provided, the default error handler is
      * returned.
      */
-    virtual const std::function<HttpResponsePtr(HttpStatusCode)>
-        &getCustomErrorHandler() const = 0;
+    virtual const std::function<HttpResponsePtr(HttpStatusCode,
+                                                const HttpRequestPtr &req)> &
+    getCustomErrorHandler() const = 0;
 
     /// Get the plugin object registered in the framework
     /**
@@ -307,7 +333,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
                          [Check Method]---------------->[405]----------->+
                                |                                         |
                                v                                         |
-                           [Filters]------->[Filter callback]----------->+
+                     [Filters/Middlewares]------>[Filter callback]------>+
                                |                                         |
                                v             Y                           |
                       [Is OPTIONS method?]------------->[200]----------->+
@@ -320,6 +346,9 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
                                |                                         |
                                v                                         |
       Post-handling join point o---------------------------------------->+
+                               |                                         |
+                               v                                         |
+                    [Middlewares post logic]--->[Middleware callback]--->+
 
       @endcode
      *
@@ -330,7 +359,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
 
     /// Register an advice called before routing
     /**
-     * @param advice is called after all the synchronous advices return
+     * @param advice is called after all the synchronous advice return
      * nullptr and before the request is routed to any handler. The parameters
      * of the advice are same as those of the doFilter method of the Filter
      * class.
@@ -353,7 +382,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     /// Register an advice called after routing
     /**
      * @param advice is called immediately after the request matches a handler
-     * path and before any 'doFilter' method of filters applies. The parameters
+     * path and before any filters/middlewares applies. The parameters
      * of the advice are same as those of the doFilter method of the Filter
      * class.
      */
@@ -375,8 +404,8 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     /// Register an advice called before the request is handled
     /**
      * @param advice is called immediately after the request is approved by all
-     * filters and before it is handled. The parameters of the advice are
-     * same as those of the doFilter method of the Filter class.
+     * filters/middlewares and before it is handled. The parameters of the
+     * advice are same as those of the doFilter method of the Filter class.
      */
     virtual HttpAppFramework &registerPreHandlingAdvice(
         const std::function<void(const HttpRequestPtr &,
@@ -426,14 +455,14 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
 
     /// Load the configuration file with json format.
     /**
-     * @param filename the configuration file
+     * @param fileName the configuration file
      */
     virtual HttpAppFramework &loadConfigFile(
         const std::string &fileName) noexcept(false) = 0;
 
     /// Load the configuration from a Json::Value Object.
     /**
-     * @param Json::Value Object containing the configuration.
+     * @param data Json::Value Object containing the configuration.
      * @note Please refer to the configuration file for the content of the json
      * object.
      */
@@ -442,7 +471,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
 
     /// Load the configuration from a Json::Value Object.
     /**
-     * @param rvalue reference to a Json::Value object containing the
+     * @param data rvalue reference to a Json::Value object containing the
      * configuration.
      * @note Please refer to the configuration file for the content of the json
      * object.
@@ -457,8 +486,8 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * called.
      * @param ctrlName is the name of the controller. It includes the namespace
      * to which the controller belongs.
-     * @param filtersAndMethods is a vector containing Http methods or filter
-     * name constraints.
+     * @param constraints is a vector containing Http methods or middleware
+     names
      *
      *   Example:
      * @code
@@ -472,8 +501,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     virtual HttpAppFramework &registerHttpSimpleController(
         const std::string &pathName,
         const std::string &ctrlName,
-        const std::vector<internal::HttpConstraint> &filtersAndMethods =
-            std::vector<internal::HttpConstraint>{}) = 0;
+        const std::vector<internal::HttpConstraint> &constraints = {}) = 0;
 
     /// Register a handler into the framework.
     /**
@@ -481,7 +509,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * pathPattern, the handler indicated by the function parameter is called.
      * @param function indicates any type of callable object with a valid
      * processing interface.
-     * @param filtersAndMethods is the same as the third parameter in the above
+     * @param constraints is the same as the third parameter in the above
      * method.
      *
      *   Example:
@@ -507,8 +535,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     HttpAppFramework &registerHandler(
         const std::string &pathPattern,
         FUNCTION &&function,
-        const std::vector<internal::HttpConstraint> &filtersAndMethods =
-            std::vector<internal::HttpConstraint>{},
+        const std::vector<internal::HttpConstraint> &constraints = {},
         const std::string &handlerName = "")
     {
         LOG_TRACE << "pathPattern:" << pathPattern;
@@ -518,17 +545,16 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
         getLoop()->queueInLoop([binder]() { binder->createHandlerInstance(); });
 
         std::vector<HttpMethod> validMethods;
-        std::vector<std::string> filters;
-        for (auto const &filterOrMethod : filtersAndMethods)
+        std::vector<std::string> middlewares;
+        for (auto const &constraint : constraints)
         {
-            if (filterOrMethod.type() == internal::ConstraintType::HttpFilter)
+            if (constraint.type() == internal::ConstraintType::HttpMiddleware)
             {
-                filters.push_back(filterOrMethod.getFilterName());
+                middlewares.push_back(constraint.getMiddlewareName());
             }
-            else if (filterOrMethod.type() ==
-                     internal::ConstraintType::HttpMethod)
+            else if (constraint.type() == internal::ConstraintType::HttpMethod)
             {
-                validMethods.push_back(filterOrMethod.getHttpMethod());
+                validMethods.push_back(constraint.getHttpMethod());
             }
             else
             {
@@ -537,7 +563,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
             }
         }
         registerHttpController(
-            pathPattern, binder, validMethods, filters, handlerName);
+            pathPattern, binder, validMethods, middlewares, handlerName);
         return *this;
     }
 
@@ -551,8 +577,8 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * subexpression is sequentially mapped to a handler parameter.
      * @param function indicates any type of callable object with a valid
      * processing interface.
-     * @param filtersAndMethods is the same as the third parameter in the above
-     * method.
+     * @param constraints is the same as the third parameter in the
+     * above method.
      * @param handlerName a name for the handler.
      * @return HttpAppFramework&
      */
@@ -560,8 +586,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     HttpAppFramework &registerHandlerViaRegex(
         const std::string &regExp,
         FUNCTION &&function,
-        const std::vector<internal::HttpConstraint> &filtersAndMethods =
-            std::vector<internal::HttpConstraint>{},
+        const std::vector<internal::HttpConstraint> &constraints = {},
         const std::string &handlerName = "")
     {
         LOG_TRACE << "regex:" << regExp;
@@ -571,17 +596,16 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
             std::forward<FUNCTION>(function));
 
         std::vector<HttpMethod> validMethods;
-        std::vector<std::string> filters;
-        for (auto const &filterOrMethod : filtersAndMethods)
+        std::vector<std::string> middlewares;
+        for (auto const &constraint : constraints)
         {
-            if (filterOrMethod.type() == internal::ConstraintType::HttpFilter)
+            if (constraint.type() == internal::ConstraintType::HttpMiddleware)
             {
-                filters.push_back(filterOrMethod.getFilterName());
+                middlewares.push_back(constraint.getMiddlewareName());
             }
-            else if (filterOrMethod.type() ==
-                     internal::ConstraintType::HttpMethod)
+            else if (constraint.type() == internal::ConstraintType::HttpMethod)
             {
-                validMethods.push_back(filterOrMethod.getHttpMethod());
+                validMethods.push_back(constraint.getHttpMethod());
             }
             else
             {
@@ -590,7 +614,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
             }
         }
         registerHttpControllerViaRegex(
-            regExp, binder, validMethods, filters, handlerName);
+            regExp, binder, validMethods, middlewares, handlerName);
         return *this;
     }
 
@@ -602,7 +626,18 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     virtual HttpAppFramework &registerWebSocketController(
         const std::string &pathName,
         const std::string &ctrlName,
-        const std::vector<internal::HttpConstraint> &filtersAndMethods =
+        const std::vector<internal::HttpConstraint> &constraints = {}) = 0;
+
+    /// Register a WebSocketController into the framework.
+    /**
+     * The parameters of this method are the same as those in the
+     * registerHttpSimpleController() method but using regular
+     * expression string for path.
+     */
+    virtual HttpAppFramework &registerWebSocketControllerRegex(
+        const std::string &regExp,
+        const std::string &ctrlName,
+        const std::vector<internal::HttpConstraint> &constraints =
             std::vector<internal::HttpConstraint>{}) = 0;
 
     /// Register controller objects created and initialized by the user
@@ -666,13 +701,31 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
         return *this;
     }
 
+    /// Register middleware objects created and initialized by the user
+    /**
+     * This method is similar to the above method.
+     */
+    template <typename T>
+    HttpAppFramework &registerMiddleware(
+        const std::shared_ptr<T> &middlewarePtr)
+    {
+        static_assert(std::is_base_of<HttpMiddlewareBase, T>::value,
+                      "Error! Only middleware objects can be registered here");
+        static_assert(!T::isAutoCreation,
+                      "Middleware created and initialized "
+                      "automatically by drogon cannot be "
+                      "registered here");
+        DrClassMap::setSingleInstance(middlewarePtr);
+        return *this;
+    }
+
     /// Register a default handler into the framework when no handler matches
     /// the request. If set, it is executed if the static file router does
     /// not find any file corresponding to the request. Thus it replaces
     /// the default 404 not found response.
     /**
-     * @param function indicates any type of callable object with a valid
-     * processing interface.
+     * @param handler function indicates any type of callable object with
+     * a valid processing interface.
      */
     virtual HttpAppFramework &setDefaultHandler(DefaultHandler handler) = 0;
 
@@ -731,8 +784,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * pattern of the handler;
      * The last item in std::tuple is the description of the handler.
      */
-    virtual std::vector<std::tuple<std::string, HttpMethod, std::string>>
-    getHandlersInfo() const = 0;
+    virtual std::vector<HttpHandlerInfo> getHandlersInfo() const = 0;
 
     /// Get the custom configuration defined by users in the configuration file.
     virtual const Json::Value &getCustomConfig() const = 0;
@@ -764,6 +816,15 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
         const std::vector<std::pair<std::string, std::string>>
             &sslConfCmds) = 0;
 
+    /// Reload the global cert file and private key file for https server
+    /// Note: The goal of this method is not to make the framework
+    /// use the new SSL path, but rather to reload the new content
+    /// from the old path while the framework is still running.
+    /// Typically, when our SSL is about to expire,
+    /// we need to reload the SSL. The purpose of this function
+    /// is to use the new SSL certificate without stopping the framework.
+    virtual HttpAppFramework &reloadSSLFiles() = 0;
+
     /// Add plugins
     /**
      * @param configs The plugins array
@@ -793,8 +854,9 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * @param keyFile specify the cert file and the private key file for this
      * listener. If they are empty, the global configuration set by the above
      * method is used.
-     * @param useOldTLS If true, the TLS1.0/1.1 are enabled for HTTPS
+     * @param useOldTLS if true, the TLS1.0/1.1 are enabled for HTTPS
      * connections.
+     * @param sslConfCmds vector of ssl configuration key/value pairs.
      *
      * @note
      * This operation can be performed by an option in the configuration file.
@@ -813,6 +875,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     /**
      * @param timeout The number of seconds which is the timeout of a session
      * @param sameSite The default value of SameSite attribute
+     * @param cookieKey The key of the session cookie
      *
      * @note
      * Session support is disabled by default.
@@ -823,7 +886,10 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      */
     virtual HttpAppFramework &enableSession(
         const size_t timeout = 0,
-        Cookie::SameSite sameSite = Cookie::SameSite::kNull) = 0;
+        Cookie::SameSite sameSite = Cookie::SameSite::kNull,
+        const std::string &cookieKey = "JSESSIONID",
+        int maxAge = -1,
+        std::function<std::string()> idGeneratorCallback = nullptr) = 0;
 
     /// A wrapper of the above method.
     /**
@@ -835,9 +901,16 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      */
     inline HttpAppFramework &enableSession(
         const std::chrono::duration<double> &timeout,
-        Cookie::SameSite sameSite = Cookie::SameSite::kNull)
+        Cookie::SameSite sameSite = Cookie::SameSite::kNull,
+        const std::string &cookieKey = "JSESSIONID",
+        int maxAge = -1,
+        std::function<std::string()> idGeneratorCallback = nullptr)
     {
-        return enableSession((size_t)timeout.count(), sameSite);
+        return enableSession((size_t)timeout.count(),
+                             sameSite,
+                             cookieKey,
+                             maxAge,
+                             idGeneratorCallback);
     }
 
     /// Register an advice called when starting a new session.
@@ -893,7 +966,8 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * extension can be accessed.
      * @param isRecursive If it is set to false, files in sub directories can't
      * be accessed.
-     * @param filters The list of filters which acting on the location.
+     * @param middlewareNames The list of middlewares which acting on the
+     * location.
      * @return HttpAppFramework&
      */
     virtual HttpAppFramework &addALocation(
@@ -903,7 +977,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
         bool isCaseSensitive = false,
         bool allowAll = true,
         bool isRecursive = true,
-        const std::vector<std::string> &filters = {}) = 0;
+        const std::vector<std::string> &middlewareNames = {}) = 0;
 
     /// Set the path to store uploaded files.
     /**
@@ -933,21 +1007,21 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     virtual HttpAppFramework &setFileTypes(
         const std::vector<std::string> &types) = 0;
 
-/// Enable supporting for dynamic views loading.
-/**
- *
- * @param libPaths is a vector that contains paths to view files.
- *
- * @param outputPath is the directory where the output source files locate. if
- * it is set to an empty string, drogon use libPaths as output paths. If the
- * path isn't prefixed with /, it is relative path of the current working
- * directory.
- *
- * @note
- * It is disabled by default.
- * This operation can be performed by an option in the configuration file.
- */
-#ifndef _WIN32
+#if !defined(_WIN32) && !TARGET_OS_IOS
+    /// Enable supporting for dynamic views loading.
+    /**
+     *
+     * @param libPaths is a vector that contains paths to view files.
+     *
+     * @param outputPath is the directory where the output source files locate.
+     * If it is set to an empty string, drogon use libPaths as output paths. If
+     * the path isn't prefixed with /, it is the relative path of the current
+     * working directory.
+     *
+     * @note
+     * It is disabled by default.
+     * This operation can be performed by an option in the configuration file.
+     */
     virtual HttpAppFramework &enableDynamicViewsLoading(
         const std::vector<std::string> &libPaths,
         const std::string &outputPath = "") = 0;
@@ -1000,11 +1074,14 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      */
     virtual HttpAppFramework &enableRelaunchOnError() = 0;
 
-    /// Set the output path of logs.
     /**
-     * @param logPath The path to logs.
-     * @param logfileBaseName The base name of log files.
+     * @brief Set the output path of logs.
+     * @param logPath The path to logs - logs to console if empty.
+     * @param logfileBaseName The base name of log files - defaults to "drogon"
+     * if empty.
      * @param logSize indicates the maximum size of a log file.
+     * @param maxFiles max count of log file - 0 = unlimited.
+     * @param useSpdlog Use spdlog for logging (if compiled-in).
      *
      * @note
      * This operation can be performed by an option in the configuration file.
@@ -1013,10 +1090,11 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
         const std::string &logPath,
         const std::string &logfileBaseName = "",
         size_t logSize = 100000000,
-        size_t maxFiles = 0) = 0;
+        size_t maxFiles = 0,
+        bool useSpdlog = false) = 0;
 
-    /// Set the log level
     /**
+     * @brief Set the log level.
      * @param level is one of TRACE, DEBUG, INFO, WARN. The Default value is
      * DEBUG.
      *
@@ -1359,7 +1437,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      */
     virtual size_t getJsonParserStackLimit() const noexcept = 0;
     /**
-     * @brief This method is to enable or disable the unicode escaping (\u) in
+     * @brief This method is to enable or disable the unicode escaping (\\u) in
      * the json string of HTTP responses or requests. it works (disable
      * successfully) when the version of JsonCpp >= 1.9.3, the unicode escaping
      * is enabled by default.
@@ -1391,19 +1469,19 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      *
      * @return std::pair<size_t, std::string>
      */
-    virtual const std::pair<unsigned int, std::string>
-        &getFloatPrecisionInJson() const noexcept = 0;
+    virtual const std::pair<unsigned int, std::string> &
+    getFloatPrecisionInJson() const noexcept = 0;
     /// Create a database client
     /**
      * @param dbType The database type is one of
      * "postgresql","mysql","sqlite3".
      * @param host IP or host name.
      * @param port The port on which the database server is listening.
-     * @databaseName Database name
+     * @param databaseName Database name
      * @param userName User name
      * @param password Password for the database server
      * @param connectionNum The number of connections to the database server.
-     * It's valid only if @param isFast is false.
+     * It's valid only if @p isFast is false.
      * @param filename The file name of sqlite3 database file.
      * @param name The client name.
      * @param isFast Indicates if the client is a fast database client.
@@ -1414,20 +1492,22 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      * @note
      * This operation can be performed by an option in the configuration file.
      */
-    virtual HttpAppFramework &createDbClient(
-        const std::string &dbType,
-        const std::string &host,
-        const unsigned short port,
-        const std::string &databaseName,
-        const std::string &userName,
-        const std::string &password,
-        const size_t connectionNum = 1,
-        const std::string &filename = "",
-        const std::string &name = "default",
-        const bool isFast = false,
-        const std::string &characterSet = "",
-        double timeout = -1.0,
-        const bool autoBatch = false) = 0;
+    [[deprecated("Use addDbClient() instead")]] virtual HttpAppFramework &
+    createDbClient(const std::string &dbType,
+                   const std::string &host,
+                   unsigned short port,
+                   const std::string &databaseName,
+                   const std::string &userName,
+                   const std::string &password,
+                   size_t connectionNum = 1,
+                   const std::string &filename = "",
+                   const std::string &name = "default",
+                   bool isFast = false,
+                   const std::string &characterSet = "",
+                   double timeout = -1.0,
+                   bool autoBatch = false) = 0;
+
+    virtual HttpAppFramework &addDbClient(const orm::DbConfig &config) = 0;
 
     /// Create a redis client
     /**
@@ -1508,7 +1588,7 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
     /**
      * @brief handler will be called upon an exception escapes a request handler
      */
-    virtual void setExceptionHandler(ExceptionHandler handler) = 0;
+    virtual HttpAppFramework &setExceptionHandler(ExceptionHandler handler) = 0;
 
     /**
      * @brief returns the excaption handler
@@ -1529,18 +1609,46 @@ class DROGON_EXPORT HttpAppFramework : public trantor::NonCopyable
      */
     virtual int64_t getConnectionCount() const = 0;
 
+    /**
+     * @brief Set the before listen setsockopt callback.
+     *
+     * @param cb This callback will be called before the listen
+     */
+    virtual HttpAppFramework &setBeforeListenSockOptCallback(
+        std::function<void(int)> cb) = 0;
+
+    /**
+     * @brief Set the after accept setsockopt callback.
+     *
+     * @param cb This callback will be called after accept
+     */
+    virtual HttpAppFramework &setAfterAcceptSockOptCallback(
+        std::function<void(int)> cb) = 0;
+
+    /**
+     * @brief Set the client disconnect or connect callback.
+     *
+     * @param cb This callback will be called, when the client disconnect or
+     * connect
+     */
+    virtual HttpAppFramework &setConnectionCallback(
+        std::function<void(const trantor::TcpConnectionPtr &)> cb) = 0;
+
+    virtual HttpAppFramework &enableRequestStream(bool enable = true) = 0;
+    virtual bool isRequestStreamEnabled() const = 0;
+
   private:
     virtual void registerHttpController(
         const std::string &pathPattern,
         const internal::HttpBinderBasePtr &binder,
-        const std::vector<HttpMethod> &validMethods = std::vector<HttpMethod>(),
-        const std::vector<std::string> &filters = std::vector<std::string>(),
+        const std::vector<HttpMethod> &validMethods = {},
+        const std::vector<std::string> &middlewareNames = {},
         const std::string &handlerName = "") = 0;
     virtual void registerHttpControllerViaRegex(
         const std::string &regExp,
         const internal::HttpBinderBasePtr &binder,
         const std::vector<HttpMethod> &validMethods,
-        const std::vector<std::string> &filters,
+        const std::vector<std::string> &middlewareNames,
         const std::string &handlerName) = 0;
 };
 
