@@ -18,6 +18,7 @@
 #include <drogon/drogon_callbacks.h>
 #include <drogon/HttpRequest.h>
 #include <drogon/HttpResponse.h>
+#include <drogon/HttpMiddleware.h>
 #include <memory>
 
 #ifdef __cpp_impl_coroutine
@@ -30,17 +31,18 @@ namespace drogon
  * @brief The abstract base class for filters
  * For more details on the class, see the wiki site (the 'Filter' section)
  */
-class DROGON_EXPORT HttpFilterBase : public virtual DrObjectBase
+class DROGON_EXPORT HttpFilterBase : public virtual DrObjectBase,
+                                     public HttpMiddlewareBase
 {
   public:
-    /// This virtual function should be overrided in subclasses.
+    /// This virtual function should be overridden in subclasses.
     /**
      * This method is an asynchronous interface, user should return the result
      * via 'FilterCallback' or 'FilterChainCallback'.
      * @param req is the request object processed by the filter
      * @param fcb if this is called, the response object is send to the client
      * by the callback, and doFilter methods of next filters and the handler
-     * registed on the path are not called anymore.
+     * registered on the path are not called anymore.
      * @param fccb if this callback is called, the next filter's doFilter method
      * or the handler registered on the path is called.
      */
@@ -48,6 +50,24 @@ class DROGON_EXPORT HttpFilterBase : public virtual DrObjectBase
                           FilterCallback &&fcb,
                           FilterChainCallback &&fccb) = 0;
     ~HttpFilterBase() override = default;
+
+  private:
+    void invoke(const HttpRequestPtr &req,
+                MiddlewareNextCallback &&nextCb,
+                MiddlewareCallback &&mcb) final
+    {
+        auto mcbPtr = std::make_shared<MiddlewareCallback>(std::move(mcb));
+        doFilter(
+            req,
+            [mcbPtr](const HttpResponsePtr &resp) {
+                (*mcbPtr)(resp);
+            },  // fcb, intercept the response
+            [nextCb = std::move(nextCb), mcbPtr]() mutable {
+                nextCb([mcbPtr = std::move(mcbPtr)](
+                           const HttpResponsePtr &resp) { (*mcbPtr)(resp); });
+            }  // fccb, call the next middleware
+        );
+    }
 };
 
 /**
@@ -55,7 +75,7 @@ class DROGON_EXPORT HttpFilterBase : public virtual DrObjectBase
  *
  * @tparam T The type of the implementation class
  * @tparam AutoCreation The flag for automatically creating, user can set this
- * flag to false for classes that have nondefault constructors.
+ * flag to false for classes that have non-default constructors.
  */
 template <typename T, bool AutoCreation = true>
 class HttpFilter : public DrObject<T>, public HttpFilterBase
@@ -65,14 +85,6 @@ class HttpFilter : public DrObject<T>, public HttpFilterBase
     ~HttpFilter() override = default;
 };
 
-namespace internal
-{
-DROGON_EXPORT void handleException(
-    const std::exception &,
-    const HttpRequestPtr &,
-    std::function<void(const HttpResponsePtr &)> &&);
-}
-
 #ifdef __cpp_impl_coroutine
 template <typename T, bool AutoCreation = true>
 class HttpCoroFilter : public DrObject<T>, public HttpFilterBase
