@@ -29,8 +29,8 @@ class DROGON_EXPORT IntranetIpFilter : public HttpFilter<IntranetIpFilter>
     {
     }
 
-    virtual void doFilter(const HttpRequestPtr &req,
-                          FilterCallback &&fcb,
-                          FilterChainCallback &&fccb) override;
+    void doFilter(const HttpRequestPtr &req,
+                  FilterCallback &&fcb,
+                  FilterChainCallback &&fccb) override;
 };
 }  // namespace drogon
