@@ -14,9 +14,9 @@
 
 #pragma once
 
+#include "drogon/utils/Utilities.h"
 #include <drogon/exports.h>
 #include <drogon/HttpRequest.h>
-#include <map>
 #include <unordered_map>
 #include <string>
 #include <vector>
@@ -63,7 +63,7 @@ class DROGON_EXPORT HttpFile
      */
     int save() const noexcept;
 
-    /// Save the file to @param path
+    /// Save the file to @p path
     /**
      * @param path if the parameter is prefixed with "/", "./" or "../", or is
      * "." or "..", the full path is path+"/"+this->getFileName(),
@@ -123,6 +123,8 @@ class DROGON_EXPORT MultiPartParser
 {
   public:
     MultiPartParser(){};
+    MultiPartParser(const MultiPartParser &other) = default;  // Copyable
+    MultiPartParser(MultiPartParser &&other) = default;       // Movable
     ~MultiPartParser(){};
     /// Get files, This method should be called after calling the parse()
     /// method.
@@ -133,19 +135,54 @@ class DROGON_EXPORT MultiPartParser
 
     /// Get parameters, This method should be called after calling the parse ()
     /// method.
-    const std::map<std::string, std::string> &getParameters() const;
+    const SafeStringMap<std::string> &getParameters() const;
+
+    /// Get the value of an optional parameter
+    /// This method should be called after calling the parse() method.
+    template <typename T>
+    std::optional<T> getOptionalParameter(const std::string &key)
+    {
+        auto &params = getParameters();
+        auto it = params.find(key);
+        if (it != params.end())
+        {
+            try
+            {
+                return std::optional<T>(utils::fromString<T>(it->second));
+            }
+            catch (const std::exception &e)
+            {
+                LOG_ERROR << e.what();
+                return std::optional<T>{};
+            }
+        }
+        else
+        {
+            return std::optional<T>{};
+        }
+    }
+
+    /// Get the value of a parameter
+    /// This method should be called after calling the parse() method.
+    /// Note: returns a default T object if the parameter is missing
+    template <typename T>
+    T getParameter(const std::string &key)
+    {
+        return getOptionalParameter<T>(key).value_or(T{});
+    }
 
     /// Parse the http request stream to get files and parameters.
     int parse(const HttpRequestPtr &req);
 
   protected:
     std::vector<HttpFile> files_;
-    std::map<std::string, std::string> parameters_;
+    SafeStringMap<std::string> parameters_;
     int parse(const HttpRequestPtr &req,
               const char *boundaryData,
               size_t boundaryLen);
-    int parseEntity(const char *begin, const char *end);
-    HttpRequestPtr requestPtr_;
+    int parseEntity(const HttpRequestPtr &req,
+                    const char *begin,
+                    const char *end);
 };
 
 /// In order to be compatible with old interfaces
