@@ -14,9 +14,11 @@
 
 #pragma once
 
+#include <json/value.h>
 #include <memory>
 #include <string>
 #include <drogon/HttpTypes.h>
+#include <string_view>
 #include <trantor/net/InetAddress.h>
 #include <trantor/utils/NonCopyable.h>
 
@@ -112,7 +114,17 @@ class WebSocketConnection
      * @param type The message type.
      */
     virtual void send(
-        const std::string &msg,
+        std::string_view msg,
+        const WebSocketMessageType type = WebSocketMessageType::Text) = 0;
+
+    /**
+     * @brief Send a message to the peer
+     *
+     * @param json The JSON message to be sent.
+     * @param type The message type.
+     */
+    virtual void sendJson(
+        const Json::Value &json,
         const WebSocketMessageType type = WebSocketMessageType::Text) = 0;
 
     /// Return the local IP address and port number of the connection
