@@ -21,24 +21,16 @@ namespace drogon
 {
 namespace internal
 {
-template <typename T>
-struct CanConvertToString
+template <typename T, typename = void>
+struct CanConvertToString : std::false_type
 {
-    using Type = std::remove_reference_t<std::remove_cv_t<T>>;
+};
 
-  private:
-    using yes = std::true_type;
-    using no = std::false_type;
-
-    template <typename U>
-    static auto test(int) -> decltype(std::to_string(U{}), yes());
-
-    template <typename>
-    static no test(...);
-
-  public:
-    static constexpr bool value =
-        std::is_same<decltype(test<Type>(0)), yes>::value;
+template <typename T>
+struct CanConvertToString<
+    T,
+    std::void_t<decltype(std::to_string(std::declval<T>()))>> : std::true_type
+{
 };
 }  // namespace internal
 
@@ -53,21 +45,20 @@ class OStringStream
     }
 
     template <typename T>
-    std::enable_if_t<!internal::CanConvertToString<T>::value, OStringStream &>
-    operator<<(T &&value)
+    OStringStream &operator<<(T &&value)
     {
-        std::stringstream ss;
-        ss << std::forward<T>(value);
-        buffer_.append(ss.str());
-        return *this;
-    }
-
-    template <typename T>
-    std::enable_if_t<internal::CanConvertToString<T>::value, OStringStream &>
-    operator<<(T &&value)
-    {
-        buffer_.append(std::to_string(std::forward<T>(value)));
-        return *this;
+        if constexpr (internal::CanConvertToString<T>::value)
+        {
+            buffer_.append(std::to_string(std::forward<T>(value)));
+            return *this;
+        }
+        else
+        {
+            std::stringstream ss;
+            ss << std::forward<T>(value);
+            buffer_.append(ss.str());
+            return *this;
+        }
     }
 
     template <int N>
@@ -146,4 +137,4 @@ class OStringStream
   private:
     std::string buffer_;
 };
-}  // namespace drogon
\ No newline at end of file
+}  // namespace drogon
