@@ -28,6 +28,8 @@
 #include <algorithm>
 #include <filesystem>
 #include <string_view>
+#include <unordered_map>
+#include <type_traits>
 #ifdef _WIN32
 #include <time.h>
 DROGON_EXPORT char *strptime(const char *s, const char *f, struct tm *tm);
@@ -37,36 +39,45 @@ namespace drogon
 {
 namespace internal
 {
-template <typename T>
-struct CanConvertFromStringStream
+template <typename T, typename = void>
+struct CanConvertFromStringStream : std::false_type
 {
-  private:
-    using yes = std::true_type;
-    using no = std::false_type;
-
-    template <typename U>
-    static auto test(U *p, std::stringstream &&ss)
-        -> decltype((ss >> *p), yes());
-
-    template <typename>
-    static no test(...);
-
-  public:
-    static constexpr bool value =
-        std::is_same<decltype(test<T>(nullptr, std::stringstream())),
-                     yes>::value;
 };
+
+template <typename T>
+struct CanConvertFromStringStream<
+    T,
+    std::void_t<decltype(std::declval<std::stringstream &>() >>
+                         std::declval<T &>())>> : std::true_type
+{
+};
+
+template <typename T>
+struct CanConstructFromString : std::is_constructible<T, std::string>
+{
+};
+
+template <typename T>
+struct CanConvertFromString : std::is_assignable<T &, std::string>
+{
+};
+
 }  // namespace internal
 
+/**
+ * @brief Get the HTTP messages corresponding to the HTTP status codes
+ *
+ * @param code HTTP status code
+ *
+ * @return the corresponding message
+ */
+DROGON_EXPORT const std::string_view &statusCodeToString(int code);
+
 namespace utils
 {
 /// Determine if the string is an integer
-DROGON_EXPORT bool isInteger(const std::string &str);
-/// Determine if the string is an integer
 DROGON_EXPORT bool isInteger(std::string_view str);
 
-/// Determine if the string is base64 encoded
-DROGON_EXPORT bool isBase64(const std::string &str);
 /// Determine if the string is base64 encoded
 DROGON_EXPORT bool isBase64(std::string_view str);
 
@@ -79,7 +90,8 @@ DROGON_EXPORT std::string genRandomString(int length);
 
 /// Convert a binary string to hex format
 DROGON_EXPORT std::string binaryStringToHex(const unsigned char *ptr,
-                                            size_t length);
+                                            size_t length,
+                                            bool lowerCase = false);
 
 /// Get a binary string from hexadecimal format
 DROGON_EXPORT std::string hexToBinaryString(const char *ptr, size_t length);
@@ -88,8 +100,15 @@ DROGON_EXPORT std::string hexToBinaryString(const char *ptr, size_t length);
 DROGON_EXPORT std::vector<char> hexToBinaryVector(const char *ptr,
                                                   size_t length);
 
+DROGON_EXPORT void binaryStringToHex(const char *ptr,
+                                     size_t length,
+                                     char *out,
+                                     bool lowerCase = false);
+
 /// Split the string into multiple separated strings.
 /**
+ * @param str string to split
+ * @param separator element separator
  * @param acceptEmptyString if true, empty strings are accepted in the
  * result, for example, splitting the ",1,2,,3," by "," produces
  * ["","1","2","","3",""]
@@ -106,7 +125,7 @@ DROGON_EXPORT std::set<std::string> splitStringToSet(
     const std::string &separator);
 
 /// Get UUID string.
-DROGON_EXPORT std::string getUuid();
+DROGON_EXPORT std::string getUuid(bool lowercase = true);
 
 /// Get the encoded length of base64.
 constexpr size_t base64EncodedLength(size_t in_len, bool padded = true)
@@ -115,47 +134,85 @@ constexpr size_t base64EncodedLength(size_t in_len, bool padded = true)
 }
 
 /// Encode the string to base64 format.
-DROGON_EXPORT std::string base64Encode(const unsigned char *bytes_to_encode,
-                                       size_t in_len,
-                                       bool url_safe = false,
-                                       bool padded = true);
+DROGON_EXPORT void base64Encode(const unsigned char *bytesToEncode,
+                                size_t inLen,
+                                unsigned char *outputBuffer,
+                                bool urlSafe = false,
+                                bool padded = true);
+
+/// Encode the string to base64 format.
+inline std::string base64Encode(const unsigned char *bytesToEncode,
+                                size_t inLen,
+                                bool urlSafe = false,
+                                bool padded = true)
+{
+    std::string ret;
+    ret.resize(base64EncodedLength(inLen, padded));
+    base64Encode(
+        bytesToEncode, inLen, (unsigned char *)ret.data(), urlSafe, padded);
+    return ret;
+}
 
 /// Encode the string to base64 format.
 inline std::string base64Encode(std::string_view data,
-                                bool url_safe = false,
+                                bool urlSafe = false,
                                 bool padded = true)
 {
     return base64Encode((unsigned char *)data.data(),
                         data.size(),
-                        url_safe,
+                        urlSafe,
                         padded);
 }
 
 /// Encode the string to base64 format with no padding.
-inline std::string base64EncodeUnpadded(const unsigned char *bytes_to_encode,
-                                        size_t in_len,
-                                        bool url_safe = false)
+inline void base64EncodeUnpadded(const unsigned char *bytesToEncode,
+                                 size_t inLen,
+                                 unsigned char *outputBuffer,
+                                 bool urlSafe = false)
 {
-    return base64Encode(bytes_to_encode, in_len, url_safe, false);
+    base64Encode(bytesToEncode, inLen, outputBuffer, urlSafe, false);
+}
+
+/// Encode the string to base64 format with no padding.
+inline std::string base64EncodeUnpadded(const unsigned char *bytesToEncode,
+                                        size_t inLen,
+                                        bool urlSafe = false)
+{
+    return base64Encode(bytesToEncode, inLen, urlSafe, false);
 }
 
 /// Encode the string to base64 format with no padding.
 inline std::string base64EncodeUnpadded(std::string_view data,
-                                        bool url_safe = false)
+                                        bool urlSafe = false)
 {
-    return base64Encode(data, url_safe, false);
+    return base64Encode(data, urlSafe, false);
 }
 
 /// Get the decoded length of base64.
-constexpr size_t base64DecodedLength(size_t in_len)
+constexpr size_t base64DecodedLength(size_t inLen)
 {
-    return (in_len * 3) / 4;
+    return (inLen * 3) / 4;
 }
 
 /// Decode the base64 format string.
-DROGON_EXPORT std::string base64Decode(std::string_view encoded_string);
+/// Return the number of bytes written.
+DROGON_EXPORT size_t base64Decode(const char *encodedString,
+                                  size_t inLen,
+                                  unsigned char *outputBuffer);
+
+/// Decode the base64 format string.
+inline std::string base64Decode(std::string_view encodedString)
+{
+    auto inLen = encodedString.size();
+    std::string ret;
+    ret.resize(base64DecodedLength(inLen));
+    ret.resize(
+        base64Decode(encodedString.data(), inLen, (unsigned char *)ret.data()));
+    return ret;
+}
+
 DROGON_EXPORT std::vector<char> base64DecodeToVector(
-    std::string_view encoded_string);
+    std::string_view encodedString);
 
 /// Check if the string need decoding
 DROGON_EXPORT bool needUrlDecoding(const char *begin, const char *end);
@@ -214,7 +271,7 @@ inline std::string getBlake2b(const std::string &originalString)
     return getBlake2b(originalString.data(), originalString.length());
 }
 
-/// Commpress or decompress data using gzip lib.
+/// Compress or decompress data using gzip lib.
 /**
  * @param data the input data
  * @param ndata the input data length
@@ -222,7 +279,7 @@ inline std::string getBlake2b(const std::string &originalString)
 DROGON_EXPORT std::string gzipCompress(const char *data, const size_t ndata);
 DROGON_EXPORT std::string gzipDecompress(const char *data, const size_t ndata);
 
-/// Commpress or decompress data using brotli lib.
+/// Compress or decompress data using brotli lib.
 /**
  * @param data the input data
  * @param ndata the input data length
@@ -244,6 +301,12 @@ DROGON_EXPORT std::string brotliDecompress(const char *data,
 DROGON_EXPORT char *getHttpFullDate(
     const trantor::Date &date = trantor::Date::now());
 
+DROGON_EXPORT const std::string &getHttpFullDateStr(
+    const trantor::Date &date = trantor::Date::now());
+
+DROGON_EXPORT void dateToCustomFormattedString(const std::string &fmtStr,
+                                               std::string &str,
+                                               const trantor::Date &date);
 /// Get the trantor::Date object according to the http full date string
 /**
  * Returns trantor::Date(std::numeric_limits<int64_t>::max()) upon failure.
@@ -367,8 +430,9 @@ inline std::string fromNativePath(const std::wstring &strPath)
     return trantor::utils::fromNativePath(strPath);
 }
 
-/// Replace all occurances of from to to inplace
+/// Replace all occurrences of from to to inplace
 /**
+ * @param s string to alter
  * @param from string to replace
  * @param to string to replace with
  */
@@ -382,7 +446,7 @@ DROGON_EXPORT void replaceAll(std::string &s,
  * @param ptr the pointer which the random bytes are stored to
  * @param size number of bytes to generate
  *
- * @return true if generation is successfull. False otherwise
+ * @return true if generation is successful. False otherwise
  */
 DROGON_EXPORT bool secureRandomBytes(void *ptr, size_t size);
 
@@ -396,24 +460,70 @@ DROGON_EXPORT bool secureRandomBytes(void *ptr, size_t size);
 DROGON_EXPORT std::string secureRandomString(size_t size);
 
 template <typename T>
-typename std::enable_if<internal::CanConvertFromStringStream<T>::value, T>::type
-fromString(const std::string &p) noexcept(false)
+T fromString(const std::string &p) noexcept(false)
 {
-    T value{};
-    if (!p.empty())
+    if constexpr (std::is_integral<T>::value && std::is_signed<T>::value)
     {
-        std::stringstream ss(p);
-        ss >> value;
+        std::size_t pos;
+        auto v = std::stoll(p, &pos);
+        // throw if the whole string could not be parsed
+        // ("1a" should not return 1)
+        if (pos != p.size())
+            throw std::invalid_argument("Invalid value");
+        if ((v < static_cast<long long>((std::numeric_limits<T>::min)())) ||
+            (v > static_cast<long long>((std::numeric_limits<T>::max)())))
+            throw std::out_of_range("Value out of range");
+        return static_cast<T>(v);
+    }
+    else if constexpr (std::is_integral<T>::value &&
+                       (!std::is_signed<T>::value))
+    {
+        std::size_t pos;
+        auto v = std::stoull(p, &pos);
+        // throw if the whole string could not be parsed
+        // ("1a" should not return 1)
+        if (pos != p.size())
+            throw std::invalid_argument("Invalid value");
+        if (v >
+            static_cast<unsigned long long>((std::numeric_limits<T>::max)()))
+            throw std::out_of_range("Value out of range");
+        return static_cast<T>(v);
+    }
+    else if constexpr (std::is_floating_point<T>::value)
+    {
+        std::size_t pos;
+        auto v = std::stold(p, &pos);
+        // throw if the whole string could not be parsed
+        // ("1a" should not return 1)
+        if (pos != p.size())
+            throw std::invalid_argument("Invalid value");
+        if ((v < static_cast<long double>((std::numeric_limits<T>::min)())) ||
+            (v > static_cast<long double>((std::numeric_limits<T>::max)())))
+            throw std::out_of_range("Value out of range");
+        return static_cast<T>(v);
+    }
+    else if constexpr (internal::CanConvertFromStringStream<T>::value)
+    {
+        T value{};
+        if (!p.empty())
+        {
+            std::stringstream ss(p);
+            // must except in case of invalid value, not return a default value
+            // (else it returns 0 for integers if the string is empty or
+            // non-numeric)
+            ss.exceptions(std::ios_base::failbit);
+            ss >> value;
+            // throw if the whole string could not be parsed
+            // ("1a" should not return 1)
+            if (!ss.eof())
+                std::runtime_error("Bad type conversion");
+        }
+        return value;
+    }
+    else
+    {
+        throw std::runtime_error("Bad type conversion");
     }
-    return value;
-}
-
-template <typename T>
-typename std::enable_if<!(internal::CanConvertFromStringStream<T>::value),
-                        T>::type
-fromString(const std::string &) noexcept(false)
-{
-    throw std::runtime_error("Bad type conversion");
 }
 
 template <>
@@ -422,67 +532,13 @@ inline std::string fromString<std::string>(const std::string &p) noexcept(false)
     return p;
 }
 
-template <>
-inline int fromString<int>(const std::string &p) noexcept(false)
-{
-    return std::stoi(p);
-}
-
-template <>
-inline long fromString<long>(const std::string &p) noexcept(false)
-{
-    return std::stol(p);
-}
-
-template <>
-inline long long fromString<long long>(const std::string &p) noexcept(false)
-{
-    return std::stoll(p);
-}
-
-template <>
-inline unsigned long fromString<unsigned long>(const std::string &p) noexcept(
-    false)
-{
-    return std::stoul(p);
-}
-
-template <>
-inline unsigned long long fromString<unsigned long long>(
-    const std::string &p) noexcept(false)
-{
-    return std::stoull(p);
-}
-
-template <>
-inline float fromString<float>(const std::string &p) noexcept(false)
-{
-    return std::stof(p);
-}
-
-template <>
-inline double fromString<double>(const std::string &p) noexcept(false)
-{
-    return std::stod(p);
-}
-
-template <>
-inline long double fromString<long double>(const std::string &p) noexcept(false)
-{
-    return std::stold(p);
-}
-
 template <>
 inline bool fromString<bool>(const std::string &p) noexcept(false)
 {
-    if (p == "1")
-    {
-        return true;
-    }
-    if (p == "0")
-    {
-        return false;
-    }
+    if (!p.empty() && std::all_of(p.begin(), p.end(), [](unsigned char c) {
+            return std::isdigit(c);
+        }))
+        return (std::stoll(p) != 0);
     std::string l{p};
     std::transform(p.begin(), p.end(), l.begin(), [](unsigned char c) {
         return (char)tolower(c);
@@ -518,13 +574,18 @@ struct SafeStringHash
 };
 }  // namespace internal
 }  // namespace utils
+
+template <typename T>
+using SafeStringMap =
+    std::unordered_map<std::string, T, utils::internal::SafeStringHash>;
 }  // namespace drogon
 
 namespace trantor
 {
 inline LogStream &operator<<(LogStream &ls, const std::string_view &v)
 {
-    ls.append(v.data(), v.length());
+    if (!v.empty())
+        ls.append(v.data(), v.length());
     return ls;
 }
 
