@@ -30,6 +30,33 @@
 #include <pthread_np.h>
 #endif
 
+#ifdef DROGON_SPDLOG_SUPPORT
+#include <spdlog/spdlog.h>
+#include <spdlog/logger.h>
+#include <spdlog/sinks/stdout_color_sinks.h>
+#include <spdlog/sinks/rotating_file_sink.h>
+#ifdef _WIN32
+#include <spdlog/sinks/msvc_sink.h>
+// Damn antedeluvian M$ macros
+#undef min
+#undef max
+#endif
+#ifndef _WIN32
+#include <sys/wait.h>
+#include <unistd.h>
+#define os_access access
+#elif !defined(_WIN32) || defined(__MINGW32__)
+#include <sys/file.h>
+#include <unistd.h>
+#define os_access access
+#else
+#include <io.h>
+#define os_access _waccess
+#define R_OK 04
+#define W_OK 02
+#endif
+#endif
+
 using namespace drogon;
 using namespace drogon::plugin;
 
@@ -86,7 +113,117 @@ void AccessLogger::initAndStart(const Json::Value &config)
     }
     createLogFunctions(format);
     auto logPath = config.get("log_path", "").asString();
-    if (!logPath.empty())
+
+    if (config.isMember("path_exempt"))
+    {
+        if (config["path_exempt"].isArray())
+        {
+            const auto &exempts = config["path_exempt"];
+            size_t exemptsCount = exempts.size();
+            if (exemptsCount)
+            {
+                std::string regexString;
+                size_t len = 0;
+                for (const auto &exempt : exempts)
+                {
+                    assert(exempt.isString());
+                    len += exempt.size();
+                }
+                regexString.reserve((exemptsCount * (1 + 2)) - 1 + len);
+
+                const auto last = --exempts.end();
+                for (auto exempt = exempts.begin(); exempt != last; ++exempt)
+                    regexString.append("(")
+                        .append(exempt->asString())
+                        .append(")|");
+                regexString.append("(").append(last->asString()).append(")");
+
+                exemptRegex_ = std::regex(regexString);
+                regexFlag_ = true;
+            }
+        }
+        else if (config["path_exempt"].isString())
+        {
+            exemptRegex_ = std::regex(config["path_exempt"].asString());
+            regexFlag_ = true;
+        }
+        else
+        {
+            LOG_ERROR << "path_exempt must be a string or string array!";
+        }
+    }
+
+#ifdef DROGON_SPDLOG_SUPPORT
+    auto logWithSpdlog = trantor::Logger::hasSpdLogSupport() &&
+                         config.get("use_spdlog", false).asBool();
+    if (logWithSpdlog)
+    {
+        logIndex_ = config.get("log_index", 0).asInt();
+        // Do nothing if already initialized...
+        if (!trantor::Logger::getSpdLogger(logIndex_))
+        {
+            trantor::Logger::enableSpdLog(logIndex_);
+            // Get the new logger & replace its sinks with the ones of the
+            // config
+            auto logger = trantor::Logger::getSpdLogger(logIndex_);
+            std::vector<spdlog::sink_ptr> sinks;
+            while (!logPath.empty())
+            {
+                // 1. check existence of folder or try to create it
+                auto fsLogPath =
+                    std::filesystem::path(utils::toNativePath(logPath));
+                std::error_code fsErr;
+                if (!std::filesystem::create_directories(fsLogPath, fsErr) &&
+                    fsErr)
+                {
+                    LOG_ERROR << "could not create log file path";
+                    break;
+                }
+                // 2. check if we have rights to create files in the folder
+                if (os_access(fsLogPath.native().c_str(), W_OK) != 0)
+                {
+                    LOG_ERROR << "cannot create files in log folder";
+                    break;
+                }
+                std::filesystem::path fileName(
+                    config.get("log_file", "access.log").asString());
+                if (fileName.empty())
+                    fileName = "access.log";
+                else
+                    fileName.replace_extension(".log");
+                auto sizeLimit = config.get("log_size_limit", 0).asUInt64();
+                if (sizeLimit == 0)
+                    sizeLimit = config.get("size_limit", 0).asUInt64();
+                if (sizeLimit == 0)  // 0 is not allowed by this sink
+                    sizeLimit = std::numeric_limits<std::size_t>::max();
+                std::size_t maxFiles = config.get("max_files", 0).asUInt();
+                sinks.push_back(
+                    std::make_shared<spdlog::sinks::rotating_file_sink_mt>(
+                        (fsLogPath / fileName).string(),
+                        sizeLimit,
+                        // spdlog limitation
+                        std::min(maxFiles, std::size_t(20000)),
+                        false));
+                break;
+            }
+            if (sinks.empty())
+                sinks.push_back(
+                    std::make_shared<spdlog::sinks::stderr_color_sink_mt>());
+#if defined(_WIN32) && defined(_DEBUG)
+            // On Windows with debug, it may be interesting to have the logs
+            // directly in the Visual Studio / WinDbg console
+            sinks.push_back(std::make_shared<spdlog::sinks::msvc_sink_mt>());
+#endif
+            logger->sinks() = sinks;
+            // Override the pattern set in
+            // trantor::Logger::getDefaultSpdLogger() and let AccessLogger
+            // format the output
+            logger->set_pattern("%v");
+        }
+    }
+    else
+#endif
+        if (!logPath.empty())
     {
         auto fileName = config.get("log_file", "access.log").asString();
         auto extension = std::string(".log");
@@ -131,7 +268,17 @@ void AccessLogger::initAndStart(const Json::Value &config)
     drogon::app().registerPreSendingAdvice(
         [this](const drogon::HttpRequestPtr &req,
                const drogon::HttpResponsePtr &resp) {
-            logging(LOG_RAW_TO(logIndex_), req, resp);
+            if (regexFlag_)
+            {
+                if (!std::regex_match(req->path(), exemptRegex_))
+                {
+                    logging(LOG_RAW_TO(logIndex_), req, resp);
+                }
+            }
+            else
+            {
+                logging(LOG_RAW_TO(logIndex_), req, resp);
+            }
         });
 }
 
@@ -270,12 +417,12 @@ void AccessLogger::outputDate(trantor::LogStream &stream,
     {
         if (useLocalTime_)
         {
-            stream << trantor::Date::now().toCustomedFormattedStringLocal(
+            stream << trantor::Date::now().toCustomFormattedStringLocal(
                 timeFormat_, showMicroseconds_);
         }
         else
         {
-            stream << trantor::Date::now().toCustomedFormattedString(
+            stream << trantor::Date::now().toCustomFormattedString(
                 timeFormat_, showMicroseconds_);
         }
     }
@@ -301,12 +448,12 @@ void AccessLogger::outputReqDate(trantor::LogStream &stream,
     {
         if (useLocalTime_)
         {
-            stream << req->creationDate().toCustomedFormattedStringLocal(
+            stream << req->creationDate().toCustomFormattedStringLocal(
                 timeFormat_, showMicroseconds_);
         }
         else
         {
-            stream << req->creationDate().toCustomedFormattedString(
+            stream << req->creationDate().toCustomFormattedString(
                 timeFormat_, showMicroseconds_);
         }
     }
