@@ -14,9 +14,9 @@
 
 #pragma once
 
-#include <drogon/utils/string_view.h>
 #include <trantor/utils/NonCopyable.h>
 #include <string>
+#include <string_view>
 #include <stdio.h>
 
 namespace drogon
@@ -26,16 +26,19 @@ class CacheFile : public trantor::NonCopyable
   public:
     explicit CacheFile(const std::string &path, bool autoDelete = true);
     ~CacheFile();
+
     void append(const std::string &data)
     {
         append(data.data(), data.length());
     }
+
     void append(const char *data, size_t length);
-    string_view getStringView()
+
+    std::string_view getStringView()
     {
         if (data())
-            return string_view(data_, dataLength_);
-        return string_view();
+            return std::string_view(data_, dataLength_);
+        return std::string_view();
     }
 
   private:
