@@ -15,6 +15,7 @@
 #pragma once
 
 #include <drogon/orm/DbClient.h>
+#include <drogon/orm/DbConfig.h>
 #include <drogon/HttpAppFramework.h>
 #include <drogon/IOThreadStorage.h>
 #include <trantor/utils/NonCopyable.h>
@@ -29,7 +30,7 @@ namespace orm
 class DbClientManager : public trantor::NonCopyable
 {
   public:
-    void createDbClients(const std::vector<trantor::EventLoop *> &ioloops);
+    void createDbClients(const std::vector<trantor::EventLoop *> &ioLoops);
 
     DbClientPtr getDbClient(const std::string &name)
     {
@@ -46,19 +47,7 @@ class DbClientManager : public trantor::NonCopyable
         return iter->second.getThreadData();
     }
 
-    void createDbClient(const std::string &dbType,
-                        const std::string &host,
-                        const unsigned short port,
-                        const std::string &databaseName,
-                        const std::string &userName,
-                        const std::string &password,
-                        const size_t connectionNum,
-                        const std::string &filename,
-                        const std::string &name,
-                        const bool isFast,
-                        const std::string &characterSet,
-                        double timeout,
-                        const bool autoBatch);
+    void addDbClient(const DbConfig &config);
     bool areAllDbClientsAvailable() const noexcept;
 
   private:
@@ -66,13 +55,8 @@ class DbClientManager : public trantor::NonCopyable
 
     struct DbInfo
     {
-        std::string name_;
         std::string connectionInfo_;
-        ClientType dbType_;
-        bool isFast_;
-        size_t connectionNumber_;
-        double timeout_;
-        bool autoBatch_;
+        DbConfig config_;
     };
 
     std::vector<DbInfo> dbInfos_;
