@@ -18,12 +18,11 @@
 #include <drogon/config.h>
 #include <json/json.h>
 #include <functional>
-#include <limits>
 #include <memory>
-#include <mutex>
-#include <regex>
 #include <string>
 #include <vector>
+#include "SessionManager.h"
+#include "drogon/utils/Utilities.h"
 #include "impl_forwards.h"
 
 namespace trantor
@@ -33,7 +32,8 @@ class EventLoopThreadPool;
 
 namespace drogon
 {
-HttpResponsePtr defaultErrorHandler(HttpStatusCode code);
+HttpResponsePtr defaultErrorHandler(HttpStatusCode code,
+                                    const HttpRequestPtr &req);
 void defaultExceptionHandler(const std::exception &,
                              const HttpRequestPtr &,
                              std::function<void(const HttpResponsePtr &)> &&);
@@ -59,10 +59,10 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     PluginBase *getPlugin(const std::string &name) override;
     std::shared_ptr<PluginBase> getSharedPlugin(
         const std::string &name) override;
-    void addPlugins(const Json::Value &configs);
+    void addPlugins(const Json::Value &configs) override;
     void addPlugin(const std::string &name,
                    const std::vector<std::string> &dependencies,
-                   const Json::Value &config);
+                   const Json::Value &config) override;
     HttpAppFramework &addListener(
         const std::string &ip,
         uint16_t port,
@@ -84,17 +84,22 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
         override;
     HttpAppFramework &setSSLFiles(const std::string &certPath,
                                   const std::string &keyPath) override;
+
+    HttpAppFramework &reloadSSLFiles() override;
+
     void run() override;
     HttpAppFramework &registerWebSocketController(
         const std::string &pathName,
         const std::string &ctrlName,
-        const std::vector<internal::HttpConstraint> &filtersAndMethods)
-        override;
+        const std::vector<internal::HttpConstraint> &constraints) override;
+    HttpAppFramework &registerWebSocketControllerRegex(
+        const std::string &regExp,
+        const std::string &ctrlName,
+        const std::vector<internal::HttpConstraint> &constraints) override;
     HttpAppFramework &registerHttpSimpleController(
         const std::string &pathName,
         const std::string &ctrlName,
-        const std::vector<internal::HttpConstraint> &filtersAndMethods)
-        override;
+        const std::vector<internal::HttpConstraint> &constraints) override;
 
     HttpAppFramework &setCustom404Page(const HttpResponsePtr &resp,
                                        bool set404) override
@@ -108,8 +113,9 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     }
 
     HttpAppFramework &setCustomErrorHandler(
-        std::function<HttpResponsePtr(HttpStatusCode)> &&resp_generator)
-        override;
+        std::function<HttpResponsePtr(HttpStatusCode,
+                                      const HttpRequestPtr &req)>
+            &&resp_generator) override;
 
     const HttpResponsePtr &getCustom404Page();
 
@@ -133,96 +139,46 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     HttpAppFramework &registerNewConnectionAdvice(
         const std::function<bool(const trantor::InetAddress &,
                                  const trantor::InetAddress &)> &advice)
-        override
-    {
-        newConnectionAdvices_.emplace_back(advice);
-        return *this;
-    }
+        override;
 
     HttpAppFramework &registerHttpResponseCreationAdvice(
-        const std::function<void(const HttpResponsePtr &)> &advice) override
-    {
-        responseCreationAdvices_.emplace_back(advice);
-        return *this;
-    }
-
-    const std::vector<std::function<void(const HttpResponsePtr &)>>
-        &getResponseCreationAdvices() const
-    {
-        return responseCreationAdvices_;
-    }
+        const std::function<void(const HttpResponsePtr &)> &advice) override;
 
     HttpAppFramework &registerSyncAdvice(
         const std::function<HttpResponsePtr(const HttpRequestPtr &)> &advice)
-        override
-    {
-        syncAdvices_.emplace_back(advice);
-        return *this;
-    }
+        override;
 
     HttpAppFramework &registerPreRoutingAdvice(
         const std::function<void(const HttpRequestPtr &,
                                  AdviceCallback &&,
-                                 AdviceChainCallback &&)> &advice) override
-    {
-        preRoutingAdvices_.emplace_back(advice);
-        return *this;
-    }
+                                 AdviceChainCallback &&)> &advice) override;
 
     HttpAppFramework &registerPostRoutingAdvice(
         const std::function<void(const HttpRequestPtr &,
                                  AdviceCallback &&,
-                                 AdviceChainCallback &&)> &advice) override
-    {
-        postRoutingAdvices_.emplace_back(advice);
-        return *this;
-    }
+                                 AdviceChainCallback &&)> &advice) override;
 
     HttpAppFramework &registerPreHandlingAdvice(
         const std::function<void(const HttpRequestPtr &,
                                  AdviceCallback &&,
-                                 AdviceChainCallback &&)> &advice) override
-    {
-        preHandlingAdvices_.emplace_back(advice);
-        return *this;
-    }
+                                 AdviceChainCallback &&)> &advice) override;
 
     HttpAppFramework &registerPreRoutingAdvice(
-        const std::function<void(const HttpRequestPtr &)> &advice) override
-    {
-        preRoutingObservers_.emplace_back(advice);
-        return *this;
-    }
+        const std::function<void(const HttpRequestPtr &)> &advice) override;
 
     HttpAppFramework &registerPostRoutingAdvice(
-        const std::function<void(const HttpRequestPtr &)> &advice) override
-    {
-        postRoutingObservers_.emplace_back(advice);
-        return *this;
-    }
+        const std::function<void(const HttpRequestPtr &)> &advice) override;
 
     HttpAppFramework &registerPreHandlingAdvice(
-        const std::function<void(const HttpRequestPtr &)> &advice) override
-    {
-        preHandlingObservers_.emplace_back(advice);
-        return *this;
-    }
+        const std::function<void(const HttpRequestPtr &)> &advice) override;
 
     HttpAppFramework &registerPostHandlingAdvice(
         const std::function<void(const HttpRequestPtr &,
-                                 const HttpResponsePtr &)> &advice) override
-    {
-        postHandlingAdvices_.emplace_back(advice);
-        return *this;
-    }
+                                 const HttpResponsePtr &)> &advice) override;
 
     HttpAppFramework &registerPreSendingAdvice(
         const std::function<void(const HttpRequestPtr &,
-                                 const HttpResponsePtr &)> &advice) override
-    {
-        preSendingAdvices_.emplace_back(advice);
-        return *this;
-    }
+                                 const HttpResponsePtr &)> &advice) override;
 
     HttpAppFramework &setDefaultHandler(DefaultHandler handler) override;
 
@@ -230,11 +186,26 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
 
     HttpAppFramework &enableSession(
         const size_t timeout,
-        Cookie::SameSite sameSite = Cookie::SameSite::kNull) override
+        Cookie::SameSite sameSite = Cookie::SameSite::kNull,
+        const std::string &cookieKey = "JSESSIONID",
+        int maxAge = -1,
+        SessionManager::IdGeneratorCallback idGeneratorCallback =
+            nullptr) override
     {
         useSession_ = true;
         sessionTimeout_ = timeout;
         sessionSameSite_ = sameSite;
+        sessionCookieKey_ = cookieKey;
+        sessionMaxAge_ = maxAge;
+        return setSessionIdGenerator(idGeneratorCallback);
+    }
+
+    HttpAppFramework &setSessionIdGenerator(
+        SessionManager::IdGeneratorCallback idGeneratorCallback = nullptr)
+    {
+        sessionIdGeneratorCallback_ =
+            idGeneratorCallback ? idGeneratorCallback
+                                : []() { return utils::getUuid(true); };
         return *this;
     }
 
@@ -280,7 +251,7 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
         bool isCaseSensitive,
         bool allowAll,
         bool isRecursive,
-        const std::vector<std::string> &filters) override;
+        const std::vector<std::string> &middlewareNames) override;
 
     const std::string &getUploadPath() const override
     {
@@ -296,7 +267,7 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     HttpAppFramework &setUploadPath(const std::string &uploadPath) override;
     HttpAppFramework &setFileTypes(
         const std::vector<std::string> &types) override;
-#ifndef _WIN32
+#if !defined(_WIN32) && !TARGET_OS_IOS
     HttpAppFramework &enableDynamicViewsLoading(
         const std::vector<std::string> &libPaths,
         const std::string &outputPath) override;
@@ -332,7 +303,8 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     HttpAppFramework &setLogPath(const std::string &logPath,
                                  const std::string &logfileBaseName,
                                  size_t logfileSize,
-                                 size_t maxFiles) override;
+                                 size_t maxFiles,
+                                 bool useSpdlog) override;
     HttpAppFramework &setLogLevel(trantor::Logger::LogLevel level) override;
     HttpAppFramework &setLogLocalTime(bool on) override;
 
@@ -373,6 +345,11 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
         return *this;
     }
 
+    size_t getIdleConnectionTimeout() const  // could expose in base class
+    {
+        return idleConnectionTimeout_;
+    }
+
     HttpAppFramework &setKeepaliveRequestsNumber(const size_t number) override
     {
         keepaliveRequestsNumber_ = number;
@@ -462,8 +439,7 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
         return clientMaxWebSocketMessageSize_;
     }
 
-    std::vector<std::tuple<std::string, HttpMethod, std::string>>
-    getHandlersInfo() const override;
+    std::vector<HttpHandlerInfo> getHandlersInfo() const override;
 
     size_t keepaliveRequestsNumber() const
     {
@@ -561,6 +537,7 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
 
     orm::DbClientPtr getDbClient(const std::string &name) override;
     orm::DbClientPtr getFastDbClient(const std::string &name) override;
+
     HttpAppFramework &createDbClient(const std::string &dbType,
                                      const std::string &host,
                                      unsigned short port,
@@ -573,7 +550,24 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
                                      bool isFast,
                                      const std::string &characterSet,
                                      double timeout,
-                                     const bool autoBatch) override;
+                                     bool autoBatch) override;
+    // a helper method
+    void addDbClient(const std::string &dbType,
+                     const std::string &host,
+                     unsigned short port,
+                     const std::string &databaseName,
+                     const std::string &userName,
+                     const std::string &password,
+                     size_t connectionNum,
+                     const std::string &filename,
+                     const std::string &name,
+                     bool isFast,
+                     const std::string &characterSet,
+                     double timeout,
+                     bool autoBatch,
+                     std::unordered_map<std::string, std::string> options);
+    HttpAppFramework &addDbClient(const orm::DbConfig &config) override;
+
     HttpAppFramework &createRedisClient(const std::string &ip,
                                         unsigned short port,
                                         const std::string &name,
@@ -598,11 +592,6 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
         return useSendfile_;
     }
 
-    void callCallback(
-        const HttpRequestImplPtr &req,
-        const HttpResponsePtr &resp,
-        const std::function<void(const HttpResponsePtr &)> &callback);
-
     bool supportSSL() const override
     {
         return trantor::utils::tlsBackend() != "None";
@@ -619,8 +608,9 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     }
 
     bool areAllDbClientsAvailable() const noexcept override;
-    const std::function<HttpResponsePtr(HttpStatusCode)>
-        &getCustomErrorHandler() const override;
+    const std::function<HttpResponsePtr(HttpStatusCode,
+                                        const HttpRequestPtr &req)> &
+    getCustomErrorHandler() const override;
 
     bool isUsingCustomErrorHandler() const
     {
@@ -637,9 +627,10 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
         return reusePort_;
     }
 
-    void setExceptionHandler(ExceptionHandler handler) override
+    HttpAppFramework &setExceptionHandler(ExceptionHandler handler) override
     {
         exceptionHandler_ = std::move(handler);
+        return *this;
     }
 
     const ExceptionHandler &getExceptionHandler() const override
@@ -662,49 +653,49 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
         const std::string &ext,
         const std::string &mime) override;
 
-    int64_t getConnectionCount() const override
-    {
-        return connectionNum_.load(std::memory_order_relaxed);
-    }
+    // should return unsigned type!
+    int64_t getConnectionCount() const override;
+
+    // TODO: move session related codes to its own singleton class
+    void findSessionForRequest(const HttpRequestImplPtr &req);
+    HttpResponsePtr handleSessionForResponse(const HttpRequestImplPtr &req,
+                                             const HttpResponsePtr &resp);
+
+    HttpAppFramework &setBeforeListenSockOptCallback(
+        std::function<void(int)> cb) override;
+    HttpAppFramework &setAfterAcceptSockOptCallback(
+        std::function<void(int)> cb) override;
+    HttpAppFramework &setConnectionCallback(
+        std::function<void(const trantor::TcpConnectionPtr &)> cb) override;
+
+    HttpAppFramework &enableRequestStream(bool enable) override;
+    bool isRequestStreamEnabled() const override;
 
   private:
     void registerHttpController(const std::string &pathPattern,
                                 const internal::HttpBinderBasePtr &binder,
                                 const std::vector<HttpMethod> &validMethods,
-                                const std::vector<std::string> &filters,
+                                const std::vector<std::string> &middlewareNames,
                                 const std::string &handlerName) override;
     void registerHttpControllerViaRegex(
         const std::string &regExp,
         const internal::HttpBinderBasePtr &binder,
         const std::vector<HttpMethod> &validMethods,
-        const std::vector<std::string> &filters,
+        const std::vector<std::string> &middlewareNames,
         const std::string &handlerName) override;
-    void onAsyncRequest(
-        const HttpRequestImplPtr &req,
-        std::function<void(const HttpResponsePtr &)> &&callback);
-    void onNewWebsockRequest(
-        const HttpRequestImplPtr &req,
-        std::function<void(const HttpResponsePtr &)> &&callback,
-        const WebSocketConnectionImplPtr &wsConnPtr);
-    void onConnection(const trantor::TcpConnectionPtr &conn);
 
-    void findSessionForRequest(const HttpRequestImplPtr &req);
-
-    // We use a uuid string as session id;
+    // We use an uuid string as session id;
     // set sessionTimeout_=0 to make location session valid forever based on
     // cookies;
     size_t sessionTimeout_{0};
     Cookie::SameSite sessionSameSite_{Cookie::SameSite::kNull};
+    std::string sessionCookieKey_{"JSESSIONID"};
+    int sessionMaxAge_{-1};
     size_t idleConnectionTimeout_{60};
     bool useSession_{false};
     std::string serverHeader_{"server: drogon/" + drogon::getVersion() +
                               "\r\n"};
 
-    std::unique_ptr<StaticFileRouter> staticFileRouterPtr_;
-    std::unique_ptr<HttpControllersRouter> httpCtrlsRouterPtr_;
-    std::unique_ptr<HttpSimpleControllersRouter> httpSimpleCtrlsRouterPtr_;
-    std::unique_ptr<WebsocketControllersRouter> websockCtrlsRouterPtr_;
-
     std::unique_ptr<ListenerManager> listenerManagerPtr_;
     std::unique_ptr<PluginsManager> pluginsManagerPtr_;
     std::unique_ptr<orm::DbClientManager> dbClientManagerPtr_;
@@ -718,7 +709,7 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     size_t threadNum_{1};
     std::unique_ptr<trantor::EventLoopThreadPool> ioLoopThreadPool_;
 
-#ifndef _WIN32
+#if !defined(_WIN32) && !TARGET_OS_IOS
     std::vector<std::string> libFilePaths_;
     std::string libFileOutputPath_;
     std::unique_ptr<SharedLibManager> sharedLibManagerPtr_;
@@ -728,15 +719,10 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     std::string sslCertPath_;
     std::string sslKeyPath_;
 
-    size_t maxConnectionNumPerIP_{0};
-    std::unordered_map<std::string, size_t> connectionsNumMap_;
-
-    int64_t maxConnectionNum_{100000};
-    std::atomic<int64_t> connectionNum_{0};
-
     bool runAsDaemon_{false};
     bool handleSigterm_{true};
     bool relaunchOnError_{false};
+    bool logWithSpdlog_{false};
     std::string logPath_;
     std::string logfileBaseName_;
     size_t logfileSize_{100000000};
@@ -760,50 +746,23 @@ class HttpAppFrameworkImpl final : public HttpAppFramework
     std::unique_ptr<SessionManager> sessionManagerPtr_;
     std::vector<AdviceStartSessionCallback> sessionStartAdvices_;
     std::vector<AdviceDestroySessionCallback> sessionDestroyAdvices_;
+    SessionManager::IdGeneratorCallback sessionIdGeneratorCallback_;
     std::shared_ptr<trantor::AsyncFileLogger> asyncFileLoggerPtr_;
     Json::Value jsonConfig_;
     Json::Value jsonRuntimeConfig_;
     HttpResponsePtr custom404_;
-    std::function<HttpResponsePtr(HttpStatusCode)> customErrorHandler_ =
-        &defaultErrorHandler;
+    std::function<HttpResponsePtr(HttpStatusCode, const HttpRequestPtr &req)>
+        customErrorHandler_ = &defaultErrorHandler;
     static InitBeforeMainFunction initFirst_;
     bool enableServerHeader_{true};
     bool enableDateHeader_{true};
     bool reusePort_{false};
     std::vector<std::function<void()>> beginningAdvices_;
-    std::vector<std::function<bool(const trantor::InetAddress &,
-                                   const trantor::InetAddress &)>>
-        newConnectionAdvices_;
-    std::vector<std::function<void(const HttpResponsePtr &)>>
-        responseCreationAdvices_;
-    std::vector<std::function<HttpResponsePtr(const HttpRequestPtr &)>>
-        syncAdvices_;
-    std::vector<std::function<void(const HttpRequestPtr &,
-                                   AdviceCallback &&,
-                                   AdviceChainCallback &&)>>
-        preRoutingAdvices_;
-    std::vector<std::function<void(const HttpRequestPtr &,
-                                   AdviceCallback &&,
-                                   AdviceChainCallback &&)>>
-        postRoutingAdvices_;
-    std::vector<std::function<void(const HttpRequestPtr &,
-                                   AdviceCallback &&,
-                                   AdviceChainCallback &&)>>
-        preHandlingAdvices_;
-    std::vector<
-        std::function<void(const HttpRequestPtr &, const HttpResponsePtr &)>>
-        postHandlingAdvices_;
-    std::vector<
-        std::function<void(const HttpRequestPtr &, const HttpResponsePtr &)>>
-        preSendingAdvices_;
-    std::vector<std::function<void(const HttpRequestPtr &)>>
-        preRoutingObservers_;
-    std::vector<std::function<void(const HttpRequestPtr &)>>
-        postRoutingObservers_;
-    std::vector<std::function<void(const HttpRequestPtr &)>>
-        preHandlingObservers_;
+
     ExceptionHandler exceptionHandler_{defaultExceptionHandler};
     bool enableCompressedRequest_{false};
+
+    bool enableRequestStream_{false};
 };
 
 }  // namespace drogon
