@@ -44,23 +44,31 @@ class ListenerManager : public trantor::NonCopyable
                          &sslConfCmds = {});
     std::vector<trantor::InetAddress> getListeners() const;
     void createListeners(
-        const HttpAsyncCallback &httpCallback,
-        const WebSocketNewAsyncCallback &webSocketCallback,
-        const trantor::ConnectionCallback &connectionCallback,
-        size_t connectionTimeout,
         const std::string &globalCertFile,
         const std::string &globalKeyFile,
         const std::vector<std::pair<std::string, std::string>> &sslConfCmds,
-        const std::vector<trantor::EventLoop *> &ioLoops,
-        const std::vector<
-            std::function<HttpResponsePtr(const HttpRequestPtr &)>>
-            &syncAdvices,
-        const std::vector<std::function<void(const HttpRequestPtr &,
-                                             const HttpResponsePtr &)>>
-            &preSendingAdvices);
+        const std::vector<trantor::EventLoop *> &ioLoops);
     void startListening();
     void stopListening();
 
+    void setBeforeListenSockOptCallback(std::function<void(int)> cb)
+    {
+        beforeListenSetSockOptCallback_ = std::move(cb);
+    }
+
+    void setAfterAcceptSockOptCallback(std::function<void(int)> cb)
+    {
+        afterAcceptSetSockOptCallback_ = std::move(cb);
+    }
+
+    void setConnectionCallback(
+        std::function<void(const trantor::TcpConnectionPtr &)> cb)
+    {
+        connectionCallback_ = std::move(cb);
+    }
+
+    void reloadSSLFiles();
+
   private:
     struct ListenerInfo
     {
@@ -97,6 +105,9 @@ class ListenerManager : public trantor::NonCopyable
     // should have value when and only when on OS that one port can only be
     // listened by one thread
     std::unique_ptr<trantor::EventLoopThread> listeningThread_;
+    std::function<void(int)> beforeListenSetSockOptCallback_;
+    std::function<void(int)> afterAcceptSetSockOptCallback_;
+    std::function<void(const trantor::TcpConnectionPtr &)> connectionCallback_;
 };
 
 }  // namespace drogon
