@@ -96,26 +96,27 @@ void RealIpResolver::initAndStart(const Json::Value &config)
     }
 
     const Json::Value &trustIps = config["trust_ips"];
-    if (!trustIps.isArray())
+    if (!trustIps.isNull() && !trustIps.isArray())
     {
         throw std::runtime_error("Invalid trusted_ips. Should be array.");
     }
-    for (const auto &elem : trustIps)
+    for (const auto &ipOrCidr : trustIps)
     {
-        std::string ipOrCidr = elem.asString();
-        trustCIDRs_.emplace_back(ipOrCidr);
+        trustCIDRs_.emplace_back(ipOrCidr.asString());
     }
 
-    drogon::app().registerPreHandlingAdvice([this](const HttpRequestPtr &req) {
-        const std::string &ipHeader = req->getHeader(fromHeader_);
+    drogon::app().registerPreRoutingAdvice([this](const HttpRequestPtr &req) {
+        const auto &headers = req->headers();
+        auto ipHeaderFind = headers.find(fromHeader_);
         const trantor::InetAddress &peerAddr = req->getPeerAddr();
-        if (ipHeader.empty() || !matchCidr(peerAddr))
+        if (ipHeaderFind == headers.end() || !matchCidr(peerAddr, trustCIDRs_))
         {
             // Target header is empty, or
             // direct peer is already a non-proxy
             req->attributes()->insert(attributeKey_, peerAddr);
             return;
         }
+        const std::string &ipHeader = ipHeaderFind->second;
         // Use a header field which contains a single ip
         if (!useXForwardedFor_)
         {
@@ -137,7 +138,7 @@ void RealIpResolver::initAndStart(const Json::Value &config)
         while (!(ip = parser.getNext()).empty())
         {
             trantor::InetAddress addr = parseAddress(ip);
-            if (addr.isUnspecified() || matchCidr(addr))
+            if (addr.isUnspecified() || matchCidr(addr, trustCIDRs_))
             {
                 continue;
             }
@@ -175,9 +176,10 @@ const trantor::InetAddress &RealIpResolver::getRealAddr(
     return attributesPtr->get<trantor::InetAddress>(attributeKey_);
 }
 
-bool RealIpResolver::matchCidr(const trantor::InetAddress &addr) const
+bool RealIpResolver::matchCidr(const trantor::InetAddress &addr,
+                               const CIDRs &trustCIDRs)
 {
-    for (auto &cidr : trustCIDRs_)
+    for (const auto &cidr : trustCIDRs)
     {
         if ((addr.ipNetEndian() & cidr.mask_) == cidr.addr_)
         {
