@@ -21,7 +21,7 @@
 #include <brotli/encode.h>
 #endif
 #ifdef _WIN32
-#include <Rpc.h>
+#include <rpc.h>
 #include <direct.h>
 #include <io.h>
 #include <iomanip>
@@ -33,6 +33,7 @@
 #include <sstream>
 #include <string>
 #include <mutex>
+#include <random>
 #include <algorithm>
 #include <array>
 #include <locale>
@@ -83,12 +84,12 @@ namespace drogon
 {
 namespace utils
 {
-static const std::string base64Chars =
+static constexpr std::string_view base64Chars =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
     "abcdefghijklmnopqrstuvwxyz"
     "0123456789+/";
 
-static const std::string urlBase64Chars =
+static constexpr std::string_view urlBase64Chars =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
     "abcdefghijklmnopqrstuvwxyz"
     "0123456789-_";
@@ -127,7 +128,7 @@ class Base64CharMap
     char charMap_[256]{0};
 };
 
-const static Base64CharMap base64CharMap;
+static const Base64CharMap base64CharMap;
 
 static inline bool isBase64(unsigned char c)
 {
@@ -162,28 +163,16 @@ bool isBase64(std::string_view str)
 
 std::string genRandomString(int length)
 {
-    static const char char_space[] =
+    static const std::string_view char_space =
         "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
-    static std::once_flag once;
-    static const size_t len = strlen(char_space);
-    static const int randMax = RAND_MAX - (RAND_MAX % len);
-    std::call_once(once, []() {
-        std::srand(static_cast<unsigned int>(time(nullptr)));
-    });
+    std::uniform_int_distribution<size_t> dist(0, char_space.size() - 1);
+    thread_local std::mt19937 rng(std::random_device{}());
 
-    int i;
     std::string str;
     str.resize(length);
-
-    for (i = 0; i < length; ++i)
+    for (char &ch : str)
     {
-        int x = std::rand();
-        while (x >= randMax)
-        {
-            x = std::rand();
-        }
-        x = (x % len);
-        str[i] = char_space[x];
+        ch = char_space[dist(rng)];
     }
 
     return str;
@@ -289,31 +278,55 @@ std::string hexToBinaryString(const char *ptr, size_t length)
     return ret;
 }
 
-std::string binaryStringToHex(const unsigned char *ptr, size_t length)
+DROGON_EXPORT void binaryStringToHex(const char *ptr,
+                                     size_t length,
+                                     char *out,
+                                     bool lowerCase)
 {
-    std::string idString;
     for (size_t i = 0; i < length; ++i)
     {
         int value = (ptr[i] & 0xf0) >> 4;
         if (value < 10)
         {
-            idString.append(1, char(value + 48));
+            out[i * 2] = char(value + 48);
         }
         else
         {
-            idString.append(1, char(value + 55));
+            if (!lowerCase)
+            {
+                out[i * 2] = char(value + 55);
+            }
+            else
+            {
+                out[i * 2] = char(value + 87);
+            }
         }
 
         value = (ptr[i] & 0x0f);
         if (value < 10)
         {
-            idString.append(1, char(value + 48));
+            out[i * 2 + 1] = char(value + 48);
         }
         else
         {
-            idString.append(1, char(value + 55));
+            if (!lowerCase)
+            {
+                out[i * 2 + 1] = char(value + 55);
+            }
+            else
+            {
+                out[i * 2 + 1] = char(value + 87);
+            }
         }
     }
+}
+
+std::string binaryStringToHex(const unsigned char *ptr,
+                              size_t length,
+                              bool lowercase)
+{
+    std::string idString(length * 2, '\0');
+    binaryStringToHex((const char *)ptr, length, &idString[0], lowercase);
     return idString;
 }
 
@@ -342,7 +355,23 @@ std::set<std::string> splitStringToSet(const std::string &str,
     return ret;
 }
 
-std::string getUuid()
+inline std::string createUuidString(const char *str, size_t len, bool lowercase)
+{
+    assert(len == 16);
+    std::string uuid(36, '\0');
+    binaryStringToHex(str, 4, &uuid[0], lowercase);
+    uuid[8] = '-';
+    binaryStringToHex(str + 4, 2, &uuid[9], lowercase);
+    uuid[13] = '-';
+    binaryStringToHex(str + 6, 2, &uuid[14], lowercase);
+    uuid[18] = '-';
+    binaryStringToHex(str + 8, 2, &uuid[19], lowercase);
+    uuid[23] = '-';
+    binaryStringToHex(str + 10, 6, &uuid[24], lowercase);
+    return uuid;
+}
+
+std::string getUuid(bool lowercase)
 {
 #if USE_OSSP_UUID
     uuid_t *uuid;
@@ -352,7 +381,7 @@ std::string getUuid()
     size_t len{0};
     uuid_export(uuid, UUID_FMT_BIN, &str, &len);
     uuid_destroy(uuid);
-    std::string ret{binaryStringToHex((const unsigned char *)str, len)};
+    auto ret = createUuidString(str, len, lowercase);
     free(str);
     return ret;
 #elif defined __FreeBSD__ || defined __OpenBSD__
@@ -370,7 +399,7 @@ std::string getUuid()
     uuid_enc_be(binstr, uuid);
 #endif /* _BYTE_ORDER == _LITTLE_ENDIAN */
     delete uuid;
-    std::string ret{binaryStringToHex((const unsigned char *)binstr, 16)};
+    auto ret = createUuidString(binstr, 16, lowercase);
     free(binstr);
     return ret;
 #elif defined _WIN32
@@ -379,7 +408,7 @@ std::string getUuid()
     char tempStr[100];
     auto len = snprintf(tempStr,
                         sizeof(tempStr),
-                        "%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X",
+                        "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
                         uu.Data1,
                         uu.Data2,
                         uu.Data3,
@@ -395,37 +424,38 @@ std::string getUuid()
 #else
     uuid_t uu;
     uuid_generate(uu);
-    return binaryStringToHex(uu, 16);
+    auto uuid = createUuidString((const char *)uu, 16, lowercase);
+    return uuid;
 #endif
 }
 
-std::string base64Encode(const unsigned char *bytes_to_encode,
-                         size_t in_len,
-                         bool url_safe,
-                         bool padded)
+void base64Encode(const unsigned char *bytesToEncode,
+                  size_t inLen,
+                  unsigned char *outputBuffer,
+                  bool urlSafe,
+                  bool padded)
 {
-    std::string ret;
-    ret.reserve(base64EncodedLength(in_len, padded));
     int i = 0;
-    unsigned char char_array_3[3];
-    unsigned char char_array_4[4];
+    unsigned char charArray3[3];
+    unsigned char charArray4[4];
 
-    const std::string &charSet = url_safe ? urlBase64Chars : base64Chars;
+    const std::string_view charSet = urlSafe ? urlBase64Chars : base64Chars;
 
-    while (in_len--)
+    size_t a = 0;
+    while (inLen--)
     {
-        char_array_3[i++] = *(bytes_to_encode++);
+        charArray3[i++] = *(bytesToEncode++);
         if (i == 3)
         {
-            char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
-            char_array_4[1] = ((char_array_3[0] & 0x03) << 4) +
-                              ((char_array_3[1] & 0xf0) >> 4);
-            char_array_4[2] = ((char_array_3[1] & 0x0f) << 2) +
-                              ((char_array_3[2] & 0xc0) >> 6);
-            char_array_4[3] = char_array_3[2] & 0x3f;
+            charArray4[0] = (charArray3[0] & 0xfc) >> 2;
+            charArray4[1] =
+                ((charArray3[0] & 0x03) << 4) + ((charArray3[1] & 0xf0) >> 4);
+            charArray4[2] =
+                ((charArray3[1] & 0x0f) << 2) + ((charArray3[2] & 0xc0) >> 6);
+            charArray4[3] = charArray3[2] & 0x3f;
 
-            for (i = 0; (i < 4); ++i)
-                ret += charSet[char_array_4[i]];
+            for (i = 0; (i < 4); ++i, ++a)
+                outputBuffer[a] = charSet[charArray4[i]];
             i = 0;
         }
     }
@@ -433,59 +463,61 @@ std::string base64Encode(const unsigned char *bytes_to_encode,
     if (i)
     {
         for (int j = i; j < 3; ++j)
-            char_array_3[j] = '\0';
+            charArray3[j] = '\0';
 
-        char_array_4[0] = (char_array_3[0] & 0xfc) >> 2;
-        char_array_4[1] =
-            ((char_array_3[0] & 0x03) << 4) + ((char_array_3[1] & 0xf0) >> 4);
-        char_array_4[2] =
-            ((char_array_3[1] & 0x0f) << 2) + ((char_array_3[2] & 0xc0) >> 6);
-        char_array_4[3] = char_array_3[2] & 0x3f;
+        charArray4[0] = (charArray3[0] & 0xfc) >> 2;
+        charArray4[1] =
+            ((charArray3[0] & 0x03) << 4) + ((charArray3[1] & 0xf0) >> 4);
+        charArray4[2] =
+            ((charArray3[1] & 0x0f) << 2) + ((charArray3[2] & 0xc0) >> 6);
+        charArray4[3] = charArray3[2] & 0x3f;
 
-        for (int j = 0; (j <= i); ++j)
-            ret += charSet[char_array_4[j]];
+        for (int j = 0; (j <= i); ++j, ++a)
+            outputBuffer[a] = charSet[charArray4[j]];
 
         if (padded)
             while ((++i < 4))
-                ret += '=';
+            {
+                outputBuffer[a] = '=';
+                ++a;
+            }
     }
-    return ret;
 }
 
-std::vector<char> base64DecodeToVector(std::string_view encoded_string)
+std::vector<char> base64DecodeToVector(std::string_view encodedString)
 {
-    auto in_len = encoded_string.size();
+    auto inLen = encodedString.size();
     int i = 0;
     int in_{0};
-    char char_array_4[4], char_array_3[3];
+    char charArray4[4], charArray3[3];
     std::vector<char> ret;
-    ret.reserve(base64DecodedLength(in_len));
+    ret.reserve(base64DecodedLength(inLen));
 
-    while (in_len-- && (encoded_string[in_] != '='))
+    while (inLen-- && (encodedString[in_] != '='))
     {
-        if (!isBase64(encoded_string[in_]))
+        if (!isBase64(encodedString[in_]))
         {
             ++in_;
             continue;
         }
 
-        char_array_4[i++] = encoded_string[in_];
+        charArray4[i++] = encodedString[in_];
         ++in_;
         if (i == 4)
         {
             for (i = 0; i < 4; ++i)
             {
-                char_array_4[i] = base64CharMap.getIndex(char_array_4[i]);
+                charArray4[i] = base64CharMap.getIndex(charArray4[i]);
             }
 
-            char_array_3[0] =
-                (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
-            char_array_3[1] = ((char_array_4[1] & 0xf) << 4) +
-                              ((char_array_4[2] & 0x3c) >> 2);
-            char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];
+            charArray3[0] =
+                (charArray4[0] << 2) + ((charArray4[1] & 0x30) >> 4);
+            charArray3[1] =
+                ((charArray4[1] & 0xf) << 4) + ((charArray4[2] & 0x3c) >> 2);
+            charArray3[2] = ((charArray4[2] & 0x3) << 6) + charArray4[3];
 
             for (i = 0; (i < 3); ++i)
-                ret.push_back(char_array_3[i]);
+                ret.push_back(charArray3[i]);
             i = 0;
         }
     }
@@ -493,60 +525,59 @@ std::vector<char> base64DecodeToVector(std::string_view encoded_string)
     if (i)
     {
         for (int j = i; j < 4; ++j)
-            char_array_4[j] = 0;
+            charArray4[j] = 0;
 
         for (int j = 0; j < 4; ++j)
         {
-            char_array_4[j] = base64CharMap.getIndex(char_array_4[j]);
+            charArray4[j] = base64CharMap.getIndex(charArray4[j]);
         }
 
-        char_array_3[0] =
-            (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
-        char_array_3[1] =
-            ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
-        char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];
+        charArray3[0] = (charArray4[0] << 2) + ((charArray4[1] & 0x30) >> 4);
+        charArray3[1] =
+            ((charArray4[1] & 0xf) << 4) + ((charArray4[2] & 0x3c) >> 2);
+        charArray3[2] = ((charArray4[2] & 0x3) << 6) + charArray4[3];
 
         --i;
         for (int j = 0; (j < i); ++j)
-            ret.push_back(char_array_3[j]);
+            ret.push_back(charArray3[j]);
     }
 
     return ret;
 }
 
-std::string base64Decode(std::string_view encoded_string)
+size_t base64Decode(const char *encodedString,
+                    size_t inLen,
+                    unsigned char *outputBuffer)
 {
-    auto in_len = encoded_string.size();
     int i = 0;
     int in_{0};
-    unsigned char char_array_4[4], char_array_3[3];
-    std::string ret;
-    ret.reserve(base64DecodedLength(in_len));
+    unsigned char charArray4[4], charArray3[3];
 
-    while (in_len-- && (encoded_string[in_] != '='))
+    size_t a = 0;
+    while (inLen-- && (encodedString[in_] != '='))
     {
-        if (!isBase64(encoded_string[in_]))
+        if (!isBase64(encodedString[in_]))
         {
             ++in_;
             continue;
         }
 
-        char_array_4[i++] = encoded_string[in_];
+        charArray4[i++] = encodedString[in_];
         ++in_;
         if (i == 4)
         {
             for (i = 0; i < 4; ++i)
             {
-                char_array_4[i] = base64CharMap.getIndex(char_array_4[i]);
+                charArray4[i] = base64CharMap.getIndex(charArray4[i]);
             }
-            char_array_3[0] =
-                (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
-            char_array_3[1] = ((char_array_4[1] & 0xf) << 4) +
-                              ((char_array_4[2] & 0x3c) >> 2);
-            char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];
+            charArray3[0] =
+                (charArray4[0] << 2) + ((charArray4[1] & 0x30) >> 4);
+            charArray3[1] =
+                ((charArray4[1] & 0xf) << 4) + ((charArray4[2] & 0x3c) >> 2);
+            charArray3[2] = ((charArray4[2] & 0x3) << 6) + charArray4[3];
 
-            for (i = 0; (i < 3); ++i)
-                ret += char_array_3[i];
+            for (i = 0; (i < 3); ++i, ++a)
+                outputBuffer[a] = charArray3[i];
             i = 0;
         }
     }
@@ -554,25 +585,24 @@ std::string base64Decode(std::string_view encoded_string)
     if (i)
     {
         for (int j = i; j < 4; ++j)
-            char_array_4[j] = 0;
+            charArray4[j] = 0;
 
         for (int j = 0; j < 4; ++j)
         {
-            char_array_4[j] = base64CharMap.getIndex(char_array_4[j]);
+            charArray4[j] = base64CharMap.getIndex(charArray4[j]);
         }
 
-        char_array_3[0] =
-            (char_array_4[0] << 2) + ((char_array_4[1] & 0x30) >> 4);
-        char_array_3[1] =
-            ((char_array_4[1] & 0xf) << 4) + ((char_array_4[2] & 0x3c) >> 2);
-        char_array_3[2] = ((char_array_4[2] & 0x3) << 6) + char_array_4[3];
+        charArray3[0] = (charArray4[0] << 2) + ((charArray4[1] & 0x30) >> 4);
+        charArray3[1] =
+            ((charArray4[1] & 0xf) << 4) + ((charArray4[2] & 0x3c) >> 2);
+        charArray3[2] = ((charArray4[2] & 0x3) << 6) + charArray4[3];
 
         --i;
-        for (int j = 0; (j < i); ++j)
-            ret += char_array_3[j];
+        for (int j = 0; (j < i); ++j, ++a)
+            outputBuffer[a] = charArray3[j];
     }
 
-    return ret;
+    return a;
 }
 
 static std::string charToHex(char c)
@@ -998,9 +1028,38 @@ char *getHttpFullDate(const trantor::Date &date)
         return lastTimeString;
     }
     lastSecond = nowSecond;
-    date.toCustomedFormattedString("%a, %d %b %Y %H:%M:%S GMT",
-                                   lastTimeString,
-                                   sizeof(lastTimeString));
+    date.toCustomFormattedString("%a, %d %b %Y %H:%M:%S GMT",
+                                 lastTimeString,
+                                 sizeof(lastTimeString));
+    return lastTimeString;
+}
+
+void dateToCustomFormattedString(const std::string &fmtStr,
+                                 std::string &str,
+                                 const trantor::Date &date)
+{
+    auto nowSecond = date.microSecondsSinceEpoch() / MICRO_SECONDS_PRE_SEC;
+    time_t seconds = static_cast<time_t>(nowSecond);
+    struct tm tm_LValue = date.tmStruct();
+    std::stringstream Out;
+    Out.imbue(std::locale{"C"});
+    Out << std::put_time(&tm_LValue, fmtStr.c_str());
+    str = Out.str();
+}
+
+const std::string &getHttpFullDateStr(const trantor::Date &date)
+{
+    static thread_local int64_t lastSecond = 0;
+    static thread_local std::string lastTimeString(128, 0);
+    auto nowSecond = date.microSecondsSinceEpoch() / MICRO_SECONDS_PRE_SEC;
+    if (nowSecond == lastSecond)
+    {
+        return lastTimeString;
+    }
+    lastSecond = nowSecond;
+    dateToCustomFormattedString("%a, %d %b %Y %H:%M:%S GMT",
+                                lastTimeString,
+                                date);
     return lastTimeString;
 }
 
