@@ -57,7 +57,10 @@ void WebSocketClientImpl::createTcpClient()
         auto policy = trantor::TLSPolicy::defaultClientPolicy();
         policy->setUseOldTLS(useOldTLS_)
             .setValidate(validateCert_)
-            .setHostname(domain_);
+            .setHostname(domain_)
+            .setConfCmds(sslConfCmds_)
+            .setCertPath(clientCertPath_)
+            .setKeyPath(clientKeyPath_);
         tcpClientPtr_->enableSSL(std::move(policy));
     }
     auto thisPtr = shared_from_this();
@@ -452,6 +455,22 @@ void WebSocketClientImpl::connectToServer(
     }
 }
 
+void WebSocketClientImpl::setCertPath(const std::string &cert,
+                                      const std::string &key)
+{
+    clientCertPath_ = cert;
+    clientKeyPath_ = key;
+}
+
+void WebSocketClientImpl::addSSLConfigs(
+    const std::vector<std::pair<std::string, std::string>> &sslConfCmds)
+{
+    for (const auto &cmd : sslConfCmds)
+    {
+        sslConfCmds_.push_back(cmd);
+    }
+}
+
 WebSocketClientPtr WebSocketClient::newWebSocketClient(const std::string &ip,
                                                        uint16_t port,
                                                        bool useSSL,
