@@ -7,11 +7,11 @@ using namespace drogon;
 class BeginAdviceTest : public drogon::HttpSimpleController<BeginAdviceTest>
 {
   public:
-    virtual void asyncHandleHttpRequest(
+    void asyncHandleHttpRequest(
         const HttpRequestPtr &req,
         std::function<void(const HttpResponsePtr &)> &&callback) override;
     PATH_LIST_BEGIN
-    // list path definations here;
+    // list path definitions here;
     // PATH_ADD("/path","filter1","filter2",...);
     PATH_ADD("/test_begin_advice", Get);
 
