@@ -17,9 +17,9 @@ class CustomHeaderFilter : public HttpFilter<CustomHeaderFilter, false>
     {
     }
 
-    virtual void doFilter(const HttpRequestPtr &req,
-                          FilterCallback &&fcb,
-                          FilterChainCallback &&fccb) override;
+    void doFilter(const HttpRequestPtr &req,
+                  FilterCallback &&fcb,
+                  FilterChainCallback &&fccb) override;
 
   private:
     std::string field_;
