@@ -11,11 +11,11 @@ namespace example
 class TestController : public drogon::HttpSimpleController<TestController>
 {
   public:
-    virtual void asyncHandleHttpRequest(
+    void asyncHandleHttpRequest(
         const HttpRequestPtr &req,
         std::function<void(const HttpResponsePtr &)> &&callback) override;
     PATH_LIST_BEGIN
-    // list path definations here;
+    // list path definitions here;
     // PATH_ADD("/path","filter1","filter2",...);
     PATH_ADD("/", Get);
     PATH_ADD("/Test", "nonFilter");
