@@ -7,7 +7,7 @@ using namespace drogon;
 
 DROGON_TEST(HttpFullDateTest)
 {
-    auto str = utils::getHttpFullDate();
+    auto str = utils::getHttpFullDateStr();
     auto date = utils::getHttpDate(str);
     CHECK(utils::getHttpFullDate(date) == str);
 }
