@@ -130,8 +130,10 @@ class DROGON_EXPORT DbClient : public trantor::NonCopyable
     /// Async and nonblocking method
     /**
      * @param sql is the SQL statement to be executed;
-     * @param FUNCTION1 is usually the ResultCallback type;
-     * @param FUNCTION2 is usually the ExceptionCallback type;
+     * @tparam FUNCTION1 is usually the ResultCallback type;
+     * @tparam FUNCTION2 is usually the ExceptionCallback type;
+     * @param rCallback callback function;
+     * @param exceptCallback callback function on exception;
      * @param args are parameters that are bound to placeholders in the sql
      * parameter;
      *
@@ -212,6 +214,28 @@ class DROGON_EXPORT DbClient : public trantor::NonCopyable
             (binder << std::forward<Arguments>(args), 0)...};
         return internal::SqlAwaiter(std::move(binder));
     }
+
+    /**
+     * @brief Execute a SQL query asynchronously using coroutine support.
+     *        This overload accepts a vector of arguments to bind to the query.
+     * @tparam T The type of the elements in the vector.
+     * @param sql The SQL query string to execute.
+     * @param args A vector of arguments to bind to the query.
+     * @return A SqlAwaiter object that can be co_awaited to retrieve the query
+     * result.
+     * @note This method is only available when coroutine support is enabled.
+     */
+    template <typename T>
+    internal::SqlAwaiter execSqlCoro(const std::string &sql,
+                                     const std::vector<T> &args) noexcept
+    {
+        auto binder = *this << sql;
+        for (const auto &arg : args)
+        {
+            binder << arg;
+        }
+        return internal::SqlAwaiter(std::move(binder));
+    }
 #endif
 
     /// Streaming-like method for sql execution. For more information, see the
