@@ -33,6 +33,7 @@ enum class SortOrder
     ASC,
     DESC
 };
+
 namespace internal
 {
 template <typename T, bool hasPrimaryKey = true>
@@ -40,11 +41,13 @@ struct Traits
 {
     using type = typename T::PrimaryKeyType;
 };
+
 template <typename T>
 struct Traits<T, false>
 {
     using type = int;
 };
+
 template <typename T>
 struct has_sqlForFindingByPrimaryKey
 {
@@ -59,9 +62,9 @@ struct has_sqlForFindingByPrimaryKey
     static no test(...);
 
   public:
-    static constexpr bool value =
-        std::is_same<decltype(test<T>(0)), yes>::value;
+    static constexpr bool value = std::is_same_v<decltype(test<T>(0)), yes>;
 };
+
 template <typename T>
 struct has_sqlForDeletingByPrimaryKey
 {
@@ -77,8 +80,7 @@ struct has_sqlForDeletingByPrimaryKey
     static no test(...);
 
   public:
-    static constexpr bool value =
-        std::is_same<decltype(test<T>(0)), yes>::value;
+    static constexpr bool value = std::is_same_v<decltype(test<T>(0)), yes>;
 };
 }  // namespace internal
 
@@ -160,7 +162,7 @@ class Mapper
 
     /**
      * @brief Set limit and offset to achieve pagination.
-     * This method will override limit() and offset(), and will be overriden by
+     * This method will override limit() and offset(), and will be overridden by
      * them.
      *
      * @param page The page number
@@ -181,7 +183,7 @@ class Mapper
     using CountCallback = std::function<void(const size_t)>;
 
     using TraitsPKType = typename internal::
-        Traits<T, !std::is_same<typename T::PrimaryKeyType, void>::value>::type;
+        Traits<T, !std::is_same_v<typename T::PrimaryKeyType, void>>::type;
 
     /**
      * @brief Find a record by the primary key.
@@ -192,53 +194,48 @@ class Mapper
      */
 
     template <typename U = T>
-    inline typename std::enable_if<
-        !std::is_same<typename U::PrimaryKeyType, void>::value,
-        T>::type
-    findByPrimaryKey(const TraitsPKType &key) noexcept(false)
+    inline T findByPrimaryKey(const TraitsPKType &key) noexcept(false)
     {
-        static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
-                      "No primary key in the table!");
-        static_assert(
-            internal::has_sqlForFindingByPrimaryKey<T>::value,
-            "No function member named sqlForFindingByPrimaryKey, please "
-            "make sure that the model class is generated by the latest "
-            "version of drogon_ctl");
-        // return findOne(Criteria(T::primaryKeyName, key));
-        std::string sql = T::sqlForFindingByPrimaryKey();
-        if (forUpdate_)
+        if constexpr (!std::is_same_v<typename U::PrimaryKeyType, void>)
         {
-            sql += " for update";
+            static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
+                          "No primary key in the table!");
+            static_assert(
+                internal::has_sqlForFindingByPrimaryKey<T>::value,
+                "No function member named sqlForFindingByPrimaryKey, please "
+                "make sure that the model class is generated by the latest "
+                "version of drogon_ctl");
+            // return findOne(Criteria(T::primaryKeyName, key));
+            std::string sql = T::sqlForFindingByPrimaryKey();
+            if (forUpdate_)
+            {
+                sql += " for update";
+            }
+            clear();
+            Result r(nullptr);
+            {
+                auto binder = *client_ << std::move(sql);
+                outputPrimaryKeyToBinder(key, binder);
+                binder << Mode::Blocking;
+                binder >> [&r](const Result &result) { r = result; };
+                binder.exec();  // exec may be throw exception;
+            }
+            if (r.size() == 0)
+            {
+                throw UnexpectedRows("0 rows found");
+            }
+            else if (r.size() > 1)
+            {
+                throw UnexpectedRows("Found more than one row");
+            }
+            auto row = r[0];
+            return T(row);
         }
-        clear();
-        Result r(nullptr);
+        else
         {
-            auto binder = *client_ << std::move(sql);
-            outputPrimeryKeyToBinder(key, binder);
-            binder << Mode::Blocking;
-            binder >> [&r](const Result &result) { r = result; };
-            binder.exec();  // exec may be throw exception;
+            LOG_FATAL << "The table must have a primary key";
+            abort();
         }
-        if (r.size() == 0)
-        {
-            throw UnexpectedRows("0 rows found");
-        }
-        else if (r.size() > 1)
-        {
-            throw UnexpectedRows("Found more than one row");
-        }
-        auto row = r[0];
-        return T(row);
-    }
-
-    template <typename U = T>
-    inline typename std::enable_if<
-        std::is_same<typename U::PrimaryKeyType, void>::value,
-        T>::type
-    findByPrimaryKey(const TraitsPKType &key) noexcept(false)
-    {
-        LOG_FATAL << "The table must have a primary key";
-        abort();
     }
 
     /**
@@ -249,55 +246,49 @@ class Mapper
      * @param ecb Is called when an error occurs or a record cannot be found.
      */
     template <typename U = T>
-    inline typename std::enable_if<
-        !std::is_same<typename U::PrimaryKeyType, void>::value,
-        void>::type
-    findByPrimaryKey(const TraitsPKType &key,
-                     const SingleRowCallback &rcb,
-                     const ExceptionCallback &ecb) noexcept
+    inline void findByPrimaryKey(const TraitsPKType &key,
+                                 const SingleRowCallback &rcb,
+                                 const ExceptionCallback &ecb) noexcept
     {
-        static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
-                      "No primary key in the table!");
-        static_assert(
-            internal::has_sqlForFindingByPrimaryKey<T>::value,
-            "No function member named sqlForFindingByPrimaryKey, please "
-            "make sure that the model class is generated by the latest "
-            "version of drogon_ctl");
-        // findOne(Criteria(T::primaryKeyName, key), rcb, ecb);
-        std::string sql = T::sqlForFindingByPrimaryKey();
-        if (forUpdate_)
+        if constexpr (!std::is_same_v<typename U::PrimaryKeyType, void>)
         {
-            sql += " for update";
+            static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
+                          "No primary key in the table!");
+            static_assert(
+                internal::has_sqlForFindingByPrimaryKey<T>::value,
+                "No function member named sqlForFindingByPrimaryKey, please "
+                "make sure that the model class is generated by the latest "
+                "version of drogon_ctl");
+            // findOne(Criteria(T::primaryKeyName, key), rcb, ecb);
+            std::string sql = T::sqlForFindingByPrimaryKey();
+            if (forUpdate_)
+            {
+                sql += " for update";
+            }
+            clear();
+            auto binder = *client_ << std::move(sql);
+            outputPrimaryKeyToBinder(key, binder);
+            binder >> [ecb, rcb](const Result &r) {
+                if (r.size() == 0)
+                {
+                    ecb(UnexpectedRows("0 rows found"));
+                }
+                else if (r.size() > 1)
+                {
+                    ecb(UnexpectedRows("Found more than one row"));
+                }
+                else
+                {
+                    rcb(T(r[0]));
+                }
+            };
+            binder >> ecb;
+        }
+        else
+        {
+            LOG_FATAL << "The table must have a primary key";
+            abort();
         }
-        clear();
-        auto binder = *client_ << std::move(sql);
-        outputPrimeryKeyToBinder(key, binder);
-        binder >> [ecb, rcb](const Result &r) {
-            if (r.size() == 0)
-            {
-                ecb(UnexpectedRows("0 rows found"));
-            }
-            else if (r.size() > 1)
-            {
-                ecb(UnexpectedRows("Found more than one row"));
-            }
-            else
-            {
-                rcb(T(r[0]));
-            }
-        };
-        binder >> ecb;
-    }
-    template <typename U = T>
-    inline typename std::enable_if<
-        std::is_same<typename U::PrimaryKeyType, void>::value,
-        void>::type
-    findByPrimaryKey(const TraitsPKType &key,
-                     const SingleRowCallback &rcb,
-                     const ExceptionCallback &ecb) noexcept
-    {
-        LOG_FATAL << "The table must have a primary key";
-        abort();
     }
 
     /**
@@ -310,60 +301,56 @@ class Mapper
      * user calls the get() method of the future object.
      */
     template <typename U = T>
-    inline typename std::enable_if<
-        !std::is_same<typename U::PrimaryKeyType, void>::value,
-        std::future<T>>::type
-    findFutureByPrimaryKey(const TraitsPKType &key) noexcept
+    inline std::future<T> findFutureByPrimaryKey(
+        const TraitsPKType &key) noexcept
     {
-        static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
-                      "No primary key in the table!");
-        static_assert(
-            internal::has_sqlForFindingByPrimaryKey<T>::value,
-            "No function member named sqlForFindingByPrimaryKey, please "
-            "make sure that the model class is generated by the latest "
-            "version of drogon_ctl");
-        // return findFutureOne(Criteria(T::primaryKeyName, key));
-        std::string sql = T::sqlForFindingByPrimaryKey();
-        if (forUpdate_)
+        if constexpr (!std::is_same_v<typename U::PrimaryKeyType, void>)
         {
-            sql += " for update";
+            static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
+                          "No primary key in the table!");
+            static_assert(
+                internal::has_sqlForFindingByPrimaryKey<T>::value,
+                "No function member named sqlForFindingByPrimaryKey, please "
+                "make sure that the model class is generated by the latest "
+                "version of drogon_ctl");
+            // return findFutureOne(Criteria(T::primaryKeyName, key));
+            std::string sql = T::sqlForFindingByPrimaryKey();
+            if (forUpdate_)
+            {
+                sql += " for update";
+            }
+            clear();
+            auto binder = *client_ << std::move(sql);
+            outputPrimaryKeyToBinder(key, binder);
+
+            std::shared_ptr<std::promise<T>> prom =
+                std::make_shared<std::promise<T>>();
+            binder >> [prom](const Result &r) {
+                if (r.size() == 0)
+                {
+                    prom->set_exception(std::make_exception_ptr(
+                        UnexpectedRows("0 rows found")));
+                }
+                else if (r.size() > 1)
+                {
+                    prom->set_exception(std::make_exception_ptr(
+                        UnexpectedRows("Found more than one row")));
+                }
+                else
+                {
+                    prom->set_value(T(r[0]));
+                }
+            };
+            binder >>
+                [prom](const std::exception_ptr &e) { prom->set_exception(e); };
+            binder.exec();
+            return prom->get_future();
+        }
+        else
+        {
+            LOG_FATAL << "The table must have a primary key";
+            abort();
         }
-        clear();
-        auto binder = *client_ << std::move(sql);
-        outputPrimeryKeyToBinder(key, binder);
-
-        std::shared_ptr<std::promise<T>> prom =
-            std::make_shared<std::promise<T>>();
-        binder >> [prom](const Result &r) {
-            if (r.size() == 0)
-            {
-                prom->set_exception(
-                    std::make_exception_ptr(UnexpectedRows("0 rows found")));
-            }
-            else if (r.size() > 1)
-            {
-                prom->set_exception(std::make_exception_ptr(
-                    UnexpectedRows("Found more than one row")));
-            }
-            else
-            {
-                prom->set_value(T(r[0]));
-            }
-        };
-        binder >>
-            [prom](const std::exception_ptr &e) { prom->set_exception(e); };
-        binder.exec();
-        return prom->get_future();
-    }
-
-    template <typename U = T>
-    inline typename std::enable_if<
-        std::is_same<typename U::PrimaryKeyType, void>::value,
-        std::future<U>>::type
-    findFutureByPrimaryKey(const TraitsPKType &key) noexcept
-    {
-        LOG_FATAL << "The table must have a primary key";
-        abort();
     }
 
     /**
@@ -590,6 +577,53 @@ class Mapper
                                        const Criteria &criteria,
                                        Arguments &&...args) noexcept;
 
+    /**
+     * @brief Update some records that match the given criteria.
+     *
+     * @param colNames Columns to increment.
+     * @param criteria The criteria.
+     * @param args Specify the amount by which the columns should be
+     * incremented.
+     * @return size_t The number of updated records.
+     */
+    template <typename... Arguments>
+    size_t increment(const std::vector<std::string> &colNames,
+                     const Criteria &criteria,
+                     Arguments... args) noexcept(false);
+
+    /**
+     * @brief Asynchronously update some records that match the given criteria.
+     *
+     * @param colNames Columns to increment.
+     * @param criteria The criteria.
+     * @param rcb is called with the result.
+     * @param ecb is called when an error occurs.
+     * @param args Specify the amount by which the columns should be
+     * incremented.
+     */
+    template <typename... Arguments>
+    void increment(const std::vector<std::string> &colNames,
+                   const CountCallback &rcb,
+                   const ExceptionCallback &ecb,
+                   const Criteria &criteria,
+                   Arguments... args) noexcept;
+
+    /**
+     * @brief Asynchronously update some records that match the given criteria.
+     *
+     * @param colNames Columns to increment.
+     * @param criteria The criteria.
+     * @param args Specify the amount by which the columns should be
+     * incremented.
+     * @return std::future<size_t> The future object with which user can get the
+     * number of updated records.
+     */
+    template <typename... Arguments>
+    std::future<size_t> incrementFuture(
+        const std::vector<std::string> &colNames,
+        const Criteria &criteria,
+        Arguments... args) noexcept;
+
     /**
      * @brief Delete a record from the table.
      *
@@ -686,6 +720,7 @@ class Mapper
     size_t offset_{0};
     std::string orderByString_;
     bool forUpdate_{false};
+
     void clear()
     {
         limit_ = 0;
@@ -693,64 +728,59 @@ class Mapper
         orderByString_.clear();
         forUpdate_ = false;
     }
+
     template <typename PKType = decltype(T::primaryKeyName)>
-    typename std::enable_if<std::is_same<const std::string, PKType>::value,
-                            void>::type
-    makePrimaryKeyCriteria(std::string &sql)
+    void makePrimaryKeyCriteria(std::string &sql)
     {
-        sql += " where ";
-        sql += T::primaryKeyName;
-        sql += " = $?";
-    }
-    template <typename PKType = decltype(T::primaryKeyName)>
-    typename std::enable_if<
-        std::is_same<const std::vector<std::string>, PKType>::value,
-        void>::type
-    makePrimaryKeyCriteria(std::string &sql)
-    {
-        sql += " where ";
-        for (size_t i = 0; i < T::primaryKeyName.size(); ++i)
+        if constexpr (std::is_same_v<const std::string, PKType>)
         {
-            sql += T::primaryKeyName[i];
+            sql += " where ";
+            sql += T::primaryKeyName;
             sql += " = $?";
-            if (i < (T::primaryKeyName.size() - 1))
+        }
+        else if constexpr (std::is_same_v<const std::vector<std::string>,
+                                          PKType>)
+        {
+            sql += " where ";
+            for (size_t i = 0; i < T::primaryKeyName.size(); ++i)
             {
-                sql += " and ";
+                sql += T::primaryKeyName[i];
+                sql += " = $?";
+                if (i < (T::primaryKeyName.size() - 1))
+                {
+                    sql += " and ";
+                }
             }
         }
     }
+
     template <typename PKType = decltype(T::primaryKeyName)>
-    typename std::enable_if<std::is_same<const std::string, PKType>::value,
-                            void>::type
-    outputPrimeryKeyToBinder(const TraitsPKType &pk,
-                             internal::SqlBinder &binder)
+    void outputPrimaryKeyToBinder(const TraitsPKType &pk,
+                                  internal::SqlBinder &binder)
     {
-        binder << pk;
-    }
-    template <typename PKType = decltype(T::primaryKeyName)>
-    typename std::enable_if<
-        std::is_same<const std::vector<std::string>, PKType>::value,
-        void>::type
-    outputPrimeryKeyToBinder(const TraitsPKType &pk,
-                             internal::SqlBinder &binder)
-    {
-        tupleToBinder<typename T::PrimaryKeyType>(pk, binder);
+        if constexpr (std::is_same_v<const std::string, PKType>)
+        {
+            binder << pk;
+        }
+        else if constexpr (std::is_same_v<const std::vector<std::string>,
+                                          PKType>)
+        {
+            tupleToBinder<typename T::PrimaryKeyType>(pk, binder);
+        }
     }
 
     template <typename TP, ssize_t N = std::tuple_size<TP>::value>
-    typename std::enable_if<(N > 1), void>::type tupleToBinder(
-        const TP &t,
-        internal::SqlBinder &binder)
+    void tupleToBinder(const TP &t, internal::SqlBinder &binder)
     {
-        tupleToBinder<TP, N - 1>(t, binder);
-        binder << std::get<N - 1>(t);
-    }
-    template <typename TP, ssize_t N = std::tuple_size<TP>::value>
-    typename std::enable_if<(N == 1), void>::type tupleToBinder(
-        const TP &t,
-        internal::SqlBinder &binder)
-    {
-        binder << std::get<0>(t);
+        if constexpr (N > 1)
+        {
+            tupleToBinder<TP, N - 1>(t, binder);
+            binder << std::get<N - 1>(t);
+        }
+        else if constexpr (N == 1)
+        {
+            binder << std::get<0>(t);
+        }
     }
 
     std::string replaceSqlPlaceHolder(const std::string &sqlStr,
@@ -927,6 +957,7 @@ inline std::future<T> Mapper<T>::findFutureOne(
     binder.exec();
     return prom->get_future();
 }
+
 template <typename T>
 inline std::vector<T> Mapper<T>::findBy(const Criteria &criteria) noexcept(
     false)
@@ -978,6 +1009,7 @@ inline std::vector<T> Mapper<T>::findBy(const Criteria &criteria) noexcept(
     }
     return ret;
 }
+
 template <typename T>
 inline void Mapper<T>::findBy(const Criteria &criteria,
                               const MultipleRowsCallback &rcb,
@@ -1021,12 +1053,13 @@ inline void Mapper<T>::findBy(const Criteria &criteria,
         std::vector<T> ret;
         for (auto const &row : r)
         {
-            ret.push_back(T(row));
+            ret.emplace_back(row);
         }
-        rcb(ret);
+        rcb(std::move(ret));
     };
     binder >> ecb;
 }
+
 template <typename T>
 inline std::future<std::vector<T>> Mapper<T>::findFutureBy(
     const Criteria &criteria) noexcept
@@ -1079,22 +1112,26 @@ inline std::future<std::vector<T>> Mapper<T>::findFutureBy(
     binder.exec();
     return prom->get_future();
 }
+
 template <typename T>
 inline std::vector<T> Mapper<T>::findAll() noexcept(false)
 {
     return findBy(Criteria());
 }
+
 template <typename T>
 inline void Mapper<T>::findAll(const MultipleRowsCallback &rcb,
                                const ExceptionCallback &ecb) noexcept
 {
     findBy(Criteria(), rcb, ecb);
 }
+
 template <typename T>
 inline std::future<std::vector<T>> Mapper<T>::findFutureAll() noexcept
 {
     return findFutureBy(Criteria());
 }
+
 template <typename T>
 inline size_t Mapper<T>::count(const Criteria &criteria) noexcept(false)
 {
@@ -1119,6 +1156,7 @@ inline size_t Mapper<T>::count(const Criteria &criteria) noexcept(false)
     assert(r.size() == 1);
     return r[0][(Row::SizeType)0].as<size_t>();
 }
+
 template <typename T>
 inline void Mapper<T>::count(const Criteria &criteria,
                              const CountCallback &rcb,
@@ -1142,6 +1180,7 @@ inline void Mapper<T>::count(const Criteria &criteria,
     };
     binder >> ecb;
 }
+
 template <typename T>
 inline std::future<size_t> Mapper<T>::countFuture(
     const Criteria &criteria) noexcept
@@ -1169,6 +1208,7 @@ inline std::future<size_t> Mapper<T>::countFuture(
     binder.exec();
     return prom->get_future();
 }
+
 template <typename T>
 inline void Mapper<T>::insert(T &obj) noexcept(false)
 {
@@ -1201,6 +1241,7 @@ inline void Mapper<T>::insert(T &obj) noexcept(false)
         }
     }
 }
+
 template <typename T>
 inline void Mapper<T>::insert(const T &obj,
                               const SingleRowCallback &rcb,
@@ -1243,6 +1284,7 @@ inline void Mapper<T>::insert(const T &obj,
     };
     binder >> ecb;
 }
+
 template <typename T>
 inline std::future<T> Mapper<T>::insertFuture(const T &obj) noexcept
 {
@@ -1298,7 +1340,7 @@ template <typename T>
 inline size_t Mapper<T>::update(const T &obj) noexcept(false)
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "update ";
     sql += T::tableName;
@@ -1317,13 +1359,14 @@ inline size_t Mapper<T>::update(const T &obj) noexcept(false)
     {
         auto binder = *client_ << std::move(sql);
         obj.updateArgs(binder);
-        outputPrimeryKeyToBinder(obj.getPrimaryKey(), binder);
+        outputPrimaryKeyToBinder(obj.getPrimaryKey(), binder);
         binder << Mode::Blocking;
         binder >> [&r](const Result &result) { r = result; };
         binder.exec();  // Maybe throw exception;
     }
     return r.affectedRows();
 }
+
 template <typename T>
 template <typename... Arguments>
 size_t Mapper<T>::updateBy(const std::vector<std::string> &colNames,
@@ -1370,7 +1413,7 @@ inline void Mapper<T>::update(const T &obj,
                               const ExceptionCallback &ecb) noexcept
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "update ";
     sql += T::tableName;
@@ -1387,10 +1430,11 @@ inline void Mapper<T>::update(const T &obj,
     sql = replaceSqlPlaceHolder(sql, "$?");
     auto binder = *client_ << std::move(sql);
     obj.updateArgs(binder);
-    outputPrimeryKeyToBinder(obj.getPrimaryKey(), binder);
+    outputPrimaryKeyToBinder(obj.getPrimaryKey(), binder);
     binder >> [rcb](const Result &r) { rcb(r.affectedRows()); };
     binder >> ecb;
 }
+
 template <typename T>
 template <typename... Arguments>
 void Mapper<T>::updateBy(const std::vector<std::string> &colNames,
@@ -1432,7 +1476,7 @@ template <typename T>
 inline std::future<size_t> Mapper<T>::updateFuture(const T &obj) noexcept
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "update ";
     sql += T::tableName;
@@ -1449,7 +1493,7 @@ inline std::future<size_t> Mapper<T>::updateFuture(const T &obj) noexcept
     sql = replaceSqlPlaceHolder(sql, "$?");
     auto binder = *client_ << std::move(sql);
     obj.updateArgs(binder);
-    outputPrimeryKeyToBinder(obj.getPrimaryKey(), binder);
+    outputPrimaryKeyToBinder(obj.getPrimaryKey(), binder);
 
     std::shared_ptr<std::promise<size_t>> prom =
         std::make_shared<std::promise<size_t>>();
@@ -1458,6 +1502,7 @@ inline std::future<size_t> Mapper<T>::updateFuture(const T &obj) noexcept
     binder.exec();
     return prom->get_future();
 }
+
 template <typename T>
 template <typename... Arguments>
 inline std::future<size_t> Mapper<T>::updateFutureBy(
@@ -1499,11 +1544,165 @@ inline std::future<size_t> Mapper<T>::updateFutureBy(
     return prom->get_future();
 }
 
+template <typename T>
+template <typename... Arguments>
+inline size_t Mapper<T>::increment(const std::vector<std::string> &colNames,
+                                   const Criteria &criteria,
+                                   Arguments... args) noexcept(false)
+{
+    static_assert(sizeof...(args) > 0);
+    assert(colNames.size() == sizeof...(args));
+    clear();
+    std::string sql = "update ";
+    sql += T::tableName;
+    sql += " set ";
+
+    std::vector<const char *> temps;
+    (void)std::initializer_list<int>{(
+        [&args, &temps] {
+            args = (args < 0) ? (temps.push_back(" - $?,"), -args)
+                              : (temps.push_back(" + $?,"), args);
+        }(),
+        0)...};
+
+    for (int i = 0; i < sizeof...(args); ++i)
+    {
+        const auto &colName = colNames[i];
+        sql += colName;
+        sql += " = ";
+        sql += colName;
+        sql += temps[i];
+    }
+    sql[sql.length() - 1] = ' ';  // Replace the last ','
+
+    if (criteria)
+    {
+        sql += " where ";
+        sql += criteria.criteriaString();
+    }
+
+    sql = replaceSqlPlaceHolder(sql, "$?");
+    Result r(nullptr);
+    {
+        auto binder = *client_ << std::move(sql);
+        (void)std::initializer_list<int>{
+            (binder << std::forward<Arguments>(args), 0)...};
+        if (criteria)
+            criteria.outputArgs(binder);
+        binder << Mode::Blocking;
+        binder >> [&r](const Result &result) { r = result; };
+        binder.exec();  // Maybe throw exception;
+    }
+    return r.affectedRows();
+}
+
+template <typename T>
+template <typename... Arguments>
+inline void Mapper<T>::increment(const std::vector<std::string> &colNames,
+                                 const CountCallback &rcb,
+                                 const ExceptionCallback &ecb,
+                                 const Criteria &criteria,
+                                 Arguments... args) noexcept
+{
+    static_assert(sizeof...(args) > 0);
+    assert(colNames.size() == sizeof...(args));
+    clear();
+    std::string sql = "update ";
+    sql += T::tableName;
+    sql += " set ";
+
+    std::vector<const char *> temps;
+    (void)std::initializer_list<int>{(
+        [&args, &temps] {
+            args = args < 0 ? (temps.push_back(" - $?,"), -args)
+                            : (temps.push_back(" + $?,"), args);
+        }(),
+        0)...};
+
+    for (int i = 0; i < sizeof...(args); ++i)
+    {
+        const auto &colName = colNames[i];
+        sql += colName;
+        sql += " = ";
+        sql += colName;
+        sql += temps[i];
+    }
+    sql[sql.length() - 1] = ' ';  // Replace the last ','
+
+    if (criteria)
+    {
+        sql += " where ";
+        sql += criteria.criteriaString();
+    }
+
+    sql = replaceSqlPlaceHolder(sql, "$?");
+    auto binder = *client_ << std::move(sql);
+    (void)std::initializer_list<int>{
+        (binder << std::forward<Arguments>(args), 0)...};
+    if (criteria)
+        criteria.outputArgs(binder);
+    binder >> [rcb](const Result &r) { rcb(r.affectedRows()); };
+    binder >> ecb;
+}
+
+template <typename T>
+template <typename... Arguments>
+inline std::future<size_t> Mapper<T>::incrementFuture(
+    const std::vector<std::string> &colNames,
+    const Criteria &criteria,
+    Arguments... args) noexcept
+{
+    static_assert(sizeof...(args) > 0);
+    assert(colNames.size() == sizeof...(args));
+    clear();
+    std::string sql = "update ";
+    sql += T::tableName;
+    sql += " set ";
+
+    std::vector<const char *> temps;
+    (void)std::initializer_list<int>{(
+        [&args, &temps] {
+            args = args < 0 ? (temps.push_back(" - $?,"), -args)
+                            : (temps.push_back(" + $?,"), args);
+        }(),
+        0)...};
+
+    for (int i = 0; i < sizeof...(args); ++i)
+    {
+        const auto &colName = colNames[i];
+        sql += colName;
+        sql += " = ";
+        sql += colName;
+        sql += temps[i];
+    }
+    sql[sql.length() - 1] = ' ';  // Replace the last ','
+
+    if (criteria)
+    {
+        sql += " where ";
+        sql += criteria.criteriaString();
+    }
+
+    sql = replaceSqlPlaceHolder(sql, "$?");
+    auto binder = *client_ << std::move(sql);
+    (void)std::initializer_list<int>{
+        (binder << std::forward<Arguments>(args), 0)...};
+    if (criteria)
+        criteria.outputArgs(binder);
+
+    std::shared_ptr<std::promise<size_t>> prom =
+        std::make_shared<std::promise<size_t>>();
+    binder >> [prom](const Result &r) { prom->set_value(r.affectedRows()); };
+    binder >> [prom](const std::exception_ptr &e) { prom->set_exception(e); };
+    binder.exec();
+    return prom->get_future();
+}
+
 template <typename T>
 inline size_t Mapper<T>::deleteOne(const T &obj) noexcept(false)
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "delete from ";
     sql += T::tableName;
@@ -1516,20 +1715,21 @@ inline size_t Mapper<T>::deleteOne(const T &obj) noexcept(false)
     Result r(nullptr);
     {
         auto binder = *client_ << std::move(sql);
-        outputPrimeryKeyToBinder(obj.getPrimaryKey(), binder);
+        outputPrimaryKeyToBinder(obj.getPrimaryKey(), binder);
         binder << Mode::Blocking;
         binder >> [&r](const Result &result) { r = result; };
         binder.exec();  // Maybe throw exception;
     }
     return r.affectedRows();
 }
+
 template <typename T>
 inline void Mapper<T>::deleteOne(const T &obj,
                                  const CountCallback &rcb,
                                  const ExceptionCallback &ecb) noexcept
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "delete from ";
     sql += T::tableName;
@@ -1539,15 +1739,16 @@ inline void Mapper<T>::deleteOne(const T &obj,
 
     sql = replaceSqlPlaceHolder(sql, "$?");
     auto binder = *client_ << std::move(sql);
-    outputPrimeryKeyToBinder(obj.getPrimaryKey(), binder);
+    outputPrimaryKeyToBinder(obj.getPrimaryKey(), binder);
     binder >> [rcb](const Result &r) { rcb(r.affectedRows()); };
     binder >> ecb;
 }
+
 template <typename T>
 inline std::future<size_t> Mapper<T>::deleteFutureOne(const T &obj) noexcept
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "delete from ";
     sql += T::tableName;
@@ -1557,7 +1758,7 @@ inline std::future<size_t> Mapper<T>::deleteFutureOne(const T &obj) noexcept
 
     sql = replaceSqlPlaceHolder(sql, "$?");
     auto binder = *client_ << std::move(sql);
-    outputPrimeryKeyToBinder(obj.getPrimaryKey(), binder);
+    outputPrimaryKeyToBinder(obj.getPrimaryKey(), binder);
 
     std::shared_ptr<std::promise<size_t>> prom =
         std::make_shared<std::promise<size_t>>();
@@ -1571,7 +1772,7 @@ template <typename T>
 inline size_t Mapper<T>::deleteBy(const Criteria &criteria) noexcept(false)
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "delete from ";
     sql += T::tableName;
@@ -1596,13 +1797,14 @@ inline size_t Mapper<T>::deleteBy(const Criteria &criteria) noexcept(false)
     }
     return r.affectedRows();
 }
+
 template <typename T>
 inline void Mapper<T>::deleteBy(const Criteria &criteria,
                                 const CountCallback &rcb,
                                 const ExceptionCallback &ecb) noexcept
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "delete from ";
     sql += T::tableName;
@@ -1622,12 +1824,13 @@ inline void Mapper<T>::deleteBy(const Criteria &criteria,
     binder >> [rcb](const Result &r) { rcb(r.affectedRows()); };
     binder >> ecb;
 }
+
 template <typename T>
 inline std::future<size_t> Mapper<T>::deleteFutureBy(
     const Criteria &criteria) noexcept
 {
     clear();
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     std::string sql = "delete from ";
     sql += T::tableName;
@@ -1658,12 +1861,14 @@ inline Mapper<T> &Mapper<T>::limit(size_t limit)
     limit_ = limit;
     return *this;
 }
+
 template <typename T>
 inline Mapper<T> &Mapper<T>::offset(size_t offset)
 {
     offset_ = offset;
     return *this;
 }
+
 template <typename T>
 inline Mapper<T> &Mapper<T>::orderBy(const std::string &colName,
                                      const SortOrder &order)
@@ -1688,6 +1893,7 @@ inline Mapper<T> &Mapper<T>::orderBy(const std::string &colName,
     }
     return *this;
 }
+
 template <typename T>
 inline Mapper<T> &Mapper<T>::orderBy(size_t colIndex, const SortOrder &order)
 {
@@ -1695,18 +1901,21 @@ inline Mapper<T> &Mapper<T>::orderBy(size_t colIndex, const SortOrder &order)
     assert(!colName.empty());
     return orderBy(colName, order);
 }
+
 template <typename T>
 inline Mapper<T> &Mapper<T>::paginate(size_t page, size_t perPage)
 {
     assert(page > 0 && perPage > 0);
     return limit(perPage).offset((page - 1) * perPage);
 }
+
 template <typename T>
 inline Mapper<T> &Mapper<T>::forUpdate()
 {
     forUpdate_ = true;
     return *this;
 }
+
 template <typename T>
 inline std::string Mapper<T>::replaceSqlPlaceHolder(
     const std::string &sqlStr,
@@ -1761,7 +1970,7 @@ template <typename T>
 inline size_t Mapper<T>::deleteByPrimaryKey(
     const typename Mapper<T>::TraitsPKType &key) noexcept(false)
 {
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     static_assert(internal::has_sqlForDeletingByPrimaryKey<T>::value,
                   "No function member named sqlForDeletingByPrimaryKey, please "
@@ -1771,7 +1980,7 @@ inline size_t Mapper<T>::deleteByPrimaryKey(
     Result r(nullptr);
     {
         auto binder = *client_ << T::sqlForDeletingByPrimaryKey();
-        outputPrimeryKeyToBinder(key, binder);
+        outputPrimaryKeyToBinder(key, binder);
         binder << Mode::Blocking;
         binder >> [&r](const Result &result) { r = result; };
         binder.exec();  // exec may be throw exception;
@@ -1785,7 +1994,7 @@ inline void Mapper<T>::deleteByPrimaryKey(
     const CountCallback &rcb,
     const ExceptionCallback &ecb) noexcept
 {
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     static_assert(internal::has_sqlForDeletingByPrimaryKey<T>::value,
                   "No function member named sqlForDeletingByPrimaryKey, please "
@@ -1793,7 +2002,7 @@ inline void Mapper<T>::deleteByPrimaryKey(
                   "version of drogon_ctl");
     clear();
     auto binder = *client_ << T::sqlForDeletingByPrimaryKey();
-    outputPrimeryKeyToBinder(key, binder);
+    outputPrimaryKeyToBinder(key, binder);
     binder >>
         [rcb = std::move(rcb)](const Result &r) { rcb(r.affectedRows()); };
     binder >> ecb;
@@ -1803,7 +2012,7 @@ template <typename T>
 inline std::future<size_t> Mapper<T>::deleteFutureByPrimaryKey(
     const typename Mapper<T>::TraitsPKType &key) noexcept
 {
-    static_assert(!std::is_same<typename T::PrimaryKeyType, void>::value,
+    static_assert(!std::is_same_v<typename T::PrimaryKeyType, void>,
                   "No primary key in the table!");
     static_assert(internal::has_sqlForDeletingByPrimaryKey<T>::value,
                   "No function member named sqlForDeletingByPrimaryKey, please "
@@ -1811,7 +2020,7 @@ inline std::future<size_t> Mapper<T>::deleteFutureByPrimaryKey(
                   "version of drogon_ctl");
     clear();
     auto binder = *client_ << T::sqlForDeletingByPrimaryKey();
-    outputPrimeryKeyToBinder(key, binder);
+    outputPrimaryKeyToBinder(key, binder);
 
     std::shared_ptr<std::promise<size_t>> prom =
         std::make_shared<std::promise<size_t>>();
