@@ -21,7 +21,7 @@ using namespace drogon::orm;
 std::map<std::string, std::string> DbConnection::parseConnString(
     const std::string &connInfo)
 {
-    const static std::regex re(
+    static const std::regex re(
         R"((\w+) *= *('(?:[^\n]|\\[^\n])+'|(?:\S|\\\S)+))");
     std::smatch what;
     std::map<std::string, std::string> params;
