@@ -13,9 +13,9 @@ using namespace drogon::orm;
 class User
 {
   public:
-    const static std::string primaryKeyName;
-    const static bool hasPrimaryKey;
-    const static std::string tableName;
+    static const std::string primaryKeyName;
+    static const bool hasPrimaryKey;
+    static const std::string tableName;
 
     using PrimaryKeyType = int;
 
@@ -30,7 +30,7 @@ class User
 
     static const std::string &sqlForFindingByPrimaryKey()
     {
-        const static std::string sql =
+        static const std::string sql =
             "select * from users where user_uuid = $1";
         return sql;
     }
